@@ -17,14 +17,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import dbus, optparse, sys
+import dbus, optparse
 
 from tools import consts
 
 
 # Command line
 optparser = optparse.OptionParser(usage='Usage: %prog [options] [FILE(s)]')
-optparser.add_option('-p', '--playbin2', action='store_true', default=False, help='use the playbin2 GStreamer component (unstable)')
+optparser.add_option('-p', '--playbin', action='store_true', default=False, help='use the playbin GStreamer component instead of playbin2')
 optparser.add_option('--no-glossy-cover', action='store_true', default=False, help='disable the gloss effect applied to covers')
 optparser.add_option('--multiple-instances', action='store_true', default=False, help='start a new instance even if one is already running')
 
@@ -41,16 +41,11 @@ if not optOptions.multiple_instances:
         activeServices = dbusSession.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus').ListNames()
 
         if consts.dbusService in activeServices:
-            shouldStop      = True
-            remoteObject    = dbusSession.get_object(consts.dbusService, consts.dbusObject)
-            remoteInterface = dbus.Interface(remoteObject, consts.dbusInterface)
+            shouldStop = True
 
             # Fill the current instance with the given tracks, if any
             if len(optArgs) != 0:
-                remoteInterface.sendMsg(consts.MSG_CMD_TRACKLIST_SET, optArgs)
-
-            # Bring the current instance to the front
-            remoteInterface.sendMsg(consts.MSG_CMD_BRING_TO_FRONT, '')
+                dbus.Interface(dbusSession.get_object(consts.dbusService, '/TrackList'), consts.dbusInterface).SetTracks(optArgs, True)
     except:
         pass
 
@@ -58,11 +53,12 @@ if not optOptions.multiple_instances:
         dbusSession.close()
 
     if shouldStop:
+        import sys
         sys.exit(1)
 
 
 # Start a new instance
-import gettext, gobject, gtk, locale, signal
+import gettext, gobject, gtk, locale
 
 from tools import loadGladeFile, log, prefs
 
@@ -78,7 +74,8 @@ def realStartup():
         Perform all the initialization stuff which is not mandatory to display the window
         This function should be called within the GTK main loop, once the window has been displayed
     """
-    import atexit, dbus.mainloop.glib, gui.about, modules, webbrowser
+    import atexit, dbus.mainloop.glib, modules, signal
+
 
     def onDelete(win, event):
         """ Use our own quit sequence, that will itself destroy the window """
@@ -86,8 +83,13 @@ def realStartup():
         modules.postQuitMsg()
         return True
 
+
     def onResize(win, rect):
         """ Save the new size of the window """
+        # The first label gets more or less a third of the window's width
+        wTree.get_widget('hbox-status1').set_size_request(rect.width / 3 + 15, -1)
+
+        # Save size and maximized state
         if win.window is not None and not win.window.get_state() & gtk.gdk.WINDOW_STATE_MAXIMIZED:
             prefs.set(__name__, 'win-width',  rect.width)
             prefs.set(__name__, 'win-height', rect.height)
@@ -95,20 +97,36 @@ def realStartup():
             if prefs.get(__name__, 'view-mode', DEFAULT_VIEW_MODE)in (consts.VIEW_MODE_FULL, consts.VIEW_MODE_PLAYLIST):
                 prefs.set(__name__, 'full-win-height', rect.height)
 
+
+    def onAbout(item):
+        """ Show the about dialog box """
+        import gui.about
+        gui.about.show(window)
+
+
+    def onHelp(item):
+        """ Show help page in the web browser """
+        import webbrowser
+        webbrowser.open(consts.urlHelp)
+
+
     def onState(win, evt):
         """ Save the new state of the window """
         prefs.set(__name__, 'win-is-maximized', bool(evt.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED))
 
+
     def atExit():
         """ Final function, called just before exiting the Python interpreter """
         prefs.save()
         log.logger.info('Stopped')
 
+
     # D-Bus
     dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
 
-    # Make sure to perform a few actions before exiting the Python interpreter
+    # Register a few handlers
     atexit.register(atExit)
+    signal.signal(signal.SIGINT,  lambda sig, frame: onDelete(window, None))
     signal.signal(signal.SIGTERM, lambda sig, frame: onDelete(window, None))
 
     # GTK handlers
@@ -120,12 +138,17 @@ def realStartup():
     wTree.get_widget('menu-mode-full').connect('activate', onViewMode, consts.VIEW_MODE_FULL)
     wTree.get_widget('menu-mode-playlist').connect('activate', onViewMode, consts.VIEW_MODE_PLAYLIST)
     wTree.get_widget('menu-quit').connect('activate', lambda item: onDelete(window, None))
-    wTree.get_widget('menu-about').connect('activate', lambda item: gui.about.show(window))
-    wTree.get_widget('menu-help').connect('activate', lambda item: webbrowser.open(consts.urlHelp))
+    wTree.get_widget('menu-about').connect('activate', onAbout)
+    wTree.get_widget('menu-help').connect('activate', onHelp)
     wTree.get_widget('menu-preferences').connect('activate', lambda item: modules.showPreferences())
 
-    # Let's go
-    modules.postMsg(consts.MSG_EVT_APP_STARTED)
+    # Now we can start all modules
+    gobject.idle_add(modules.postMsg, consts.MSG_EVT_APP_STARTED)
+
+    # Immediately show the preferences the first time the application is started
+    if prefs.get(__name__, 'first-time', True):
+        prefs.set(__name__, 'first-time', False)
+        gobject.idle_add(modules.showPreferences)
 
 
 def onViewMode(item, mode):
