@@ -19,10 +19,14 @@
 # ExtListView v1.8
 #
 # v1.8:
-#   * The 'extlistview-modified' signal was not generated when calling clear() and replaceContent()
+#   * Added an __iter__ method
+#   * Don't detach the model while inserting rows to avoid unwanted scrolling
 #
 # v1.7:
-#   * (Hopefully) fixed the intermittent improper columns resizing
+#   * The 'extlistview-modified' signal was not generated when calling clear() and replaceContent()
+#   * Added selectRows(), removeRow(), and removeRows() method
+#   * Really fixed the intermittent improper columns resizing
+#   * Added the 'extlistview-selection-changed' signal
 #
 # v1.6:
 #   * Added a context menu to column headers allowing users to show/hide columns
@@ -70,6 +74,7 @@ signal_new('extlistview-modified', gtk.TreeView, SIGNAL_RUN_LAST, TYPE_NONE, ())
 signal_new('extlistview-button-pressed', gtk.TreeView, SIGNAL_RUN_LAST, TYPE_NONE, (gdk.Event, TYPE_PYOBJECT))
 signal_new('extlistview-column-visibility-changed', gtk.TreeView, SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_STRING, TYPE_BOOLEAN))
 signal_new('button-press-event', gtk.TreeViewColumn, SIGNAL_RUN_LAST, TYPE_NONE, (gdk.Event, ))
+signal_new('extlistview-selection-changed', gtk.TreeView, SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_PYOBJECT, ))
 
 
 class ExtListViewColumn(gtk.TreeViewColumn):
@@ -177,12 +182,16 @@ class ExtListView(gtk.TreeView):
         if len(dndTargets) != 0:
             self.enable_model_drag_dest(dndTargets, gdk.ACTION_DEFAULT)
 
+        # TreeView events
         self.connect('drag-begin',           self.onDragBegin)
         self.connect('drag-motion',          self.onDragMotion)
         self.connect('button-press-event',   self.onButtonPressed)
         self.connect('drag-data-received',   self.onDragDataReceived)
         self.connect('button-release-event', self.onButtonReleased)
 
+        # Selection events
+        self.selection.connect('changed', self.onSelectionChanged)
+
         # Show the list
         self.show()
 
@@ -329,6 +338,13 @@ class ExtListView(gtk.TreeView):
         self.selection.select_all()
 
 
+    def selectRows(self, paths):
+        """ Select the given rows """
+        self.unselectAll()
+        for path in paths:
+            self.selection.select_path(path)
+
+
     def getSelectedRowsCount(self):
         """ Return how many rows are currently selected """
         return self.selection.count_selected_rows()
@@ -368,6 +384,18 @@ class ExtListView(gtk.TreeView):
         return len(self.store)
 
 
+    def __iter__(self):
+        """ Iterate on all rows """
+        for row in self.store:
+            yield tuple(row)[:-1]
+
+
+    def iterAllRows(self):
+        """ Iterate on all rows """
+        for row in self.store:
+            yield tuple(row)[:-1]
+
+
     def getRow(self, rowIndex):
         """ Return the given row """
         return tuple(self.store[rowIndex])[:-1]
@@ -378,12 +406,6 @@ class ExtListView(gtk.TreeView):
         return [tuple(row)[:-1] for row in self.store]
 
 
-    def iterAllRows(self):
-        """ Iterate on all rows """
-        for row in self.store:
-            yield tuple(row)[:-1]
-
-
     def getItem(self, rowIndex, colIndex):
         """ Return the value of the given item """
         return self.store.get_value(self.store.get_iter(rowIndex), colIndex)
@@ -409,18 +431,19 @@ class ExtListView(gtk.TreeView):
         self.store.set_value(self.store.get_iter(rowIndex), colIndex, value)
 
 
-    def removeSelectedRows(self):
-        """ Remove the selected row(s) """
+    def removeRows(self, paths):
+        """ Remove the given rows """
         self.freeze_child_notify()
-        for iter in self.__getIterOnSelectedRows():
+        # We must work with iters because paths become meaningless once we start removing rows
+        for iter in [self.store.get_iter(path) for path in paths]:
+            path = self.store.get_path(iter)[0]
             # Move the mark if needed
             if self.markedRow is not None:
-                currentPath = self.store.get_path(iter)[0]
-                if   currentPath < self.markedRow:  self.markedRow -= 1
-                elif currentPath == self.markedRow: self.markedRow  = None
+                if   path < self.markedRow:  self.markedRow -= 1
+                elif path == self.markedRow: self.markedRow  = None
             # Remove the current row
-            if   self.store.remove(iter): self.set_cursor(self.store.get_path(iter))
-            elif len(self.store) != 0:    self.set_cursor(len(self.store)-1)
+            if   self.store.remove(self.store.get_iter(path)): self.set_cursor(path)
+            elif len(self.store) != 0:                         self.set_cursor(len(self.store)-1)
         self.thaw_child_notify()
         if len(self.store) == 0:
             self.set_cursor(0)
@@ -429,6 +452,16 @@ class ExtListView(gtk.TreeView):
         self.emit('extlistview-modified')
 
 
+    def removeRow(self, path):
+        """ Remove the given row """
+        self.removeRows((path, ))
+
+
+    def removeSelectedRows(self):
+        """ Remove the selected row(s) """
+        self.removeRows(self.selection.get_selected_rows()[1])
+
+
     def cropSelectedRows(self):
         """ Remove all rows but the selected ones """
         pathsList = self.selection.get_selected_rows()[1]
@@ -575,8 +608,12 @@ class ExtListView(gtk.TreeView):
                 stateClear = not (event.state & (gdk.SHIFT_MASK | gdk.CONTROL_MASK))
 
                 if stateClear and not self.selection.path_is_selected(path):
+                    # We block the 'changed' signal here because it's emitted by the default GTK handler
+                    # And we don't need to emit it multiple times because of what we're doing here
+                    self.selection.handler_block_by_func(self.onSelectionChanged)
                     self.selection.unselect_all()
                     self.selection.select_path(path)
+                    self.selection.handler_unblock_by_func(self.onSelectionChanged)
                 else:
                     retVal = (stateClear and self.getSelectedRowsCount() > 1 and self.selection.path_is_selected(path))
 
@@ -663,3 +700,8 @@ class ExtListView(gtk.TreeView):
         column.set_visible(not column.get_visible())
         self.__resizeColumns()
         self.emit('extlistview-column-visibility-changed', column.get_title(), column.get_visible())
+
+
+    def onSelectionChanged(self, selection):
+        """ The selection has changed """
+        self.emit('extlistview-selection-changed', self.getSelectedRows())
