@@ -16,10 +16,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import collections, gtk
+import gtk
 
 from gtk     import gdk
-from tools   import consts
 from gobject import signal_new, TYPE_NONE, TYPE_PYOBJECT, SIGNAL_RUN_LAST
 
 
@@ -311,6 +310,8 @@ class ExtTreeView(gtk.TreeView):
             Return a structure representing the current state of the tree
             The nameIndex parameter is the index of the value that stores rows' name
         """
+        import collections
+
         queue         = collections.deque((None,))
         expandedNodes = []
 
