@@ -16,8 +16,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import cgi, gobject, gtk, gui, modules, tools
+import gtk, gui, modules, tools
 
+from tools   import consts, icons
 from gettext import gettext as _
 
 
@@ -36,51 +37,94 @@ class Preferences:
 
     def __init__(self):
         """ Constructor """
-        self.window = gui.window.Window('Preferences.glade', 'vbox1', __name__, _('Preferences'), 390, 350)
-        # List of modules
-        toggleRdr = gtk.CellRendererToggle()
-        columns   = (('',   [(toggleRdr, gobject.TYPE_BOOLEAN)],             ROW_ENABLED,    False, True),
-                     ('',   [(gtk.CellRendererText(), gobject.TYPE_STRING)], ROW_TEXT,       True,  True),
-                     ('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf)],    ROW_ICON,       False, True),
-                     (None, [(None, gobject.TYPE_BOOLEAN)],                  ROW_UNLOADABLE, False, False),
-                     (None, [(None, gobject.TYPE_PYOBJECT)],                 ROW_INSTANCE,   False, False),
-                     (None, [(None, gobject.TYPE_PYOBJECT)],                 ROW_MODINFO,    False, False))
+        import gobject
 
-        self.list = gui.extListview.ExtListView(columns, sortable=False, useMarkup=True, canShowHideColumns=False)
+        from gui import extListview, window
+
+        self.window  = window.Window('Preferences.glade', 'vbox1', __name__, _('Preferences'), 495, 440)
+        self.currCat = consts.MODCAT_NONE
+
+        # List of modules
+        textRdr   = gtk.CellRendererText()
+        toggleRdr = gtk.CellRendererToggle()
+
+        columns   = (('',   [(toggleRdr, gobject.TYPE_BOOLEAN)],          ROW_ENABLED,    False, True),
+                     ('',   [(textRdr, gobject.TYPE_STRING)],             ROW_TEXT,       True,  True),
+                     ('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf)], ROW_ICON,       False, True),
+                     (None, [(None, gobject.TYPE_BOOLEAN)],               ROW_UNLOADABLE, False, False),
+                     (None, [(None, gobject.TYPE_PYOBJECT)],              ROW_INSTANCE,   False, False),
+                     (None, [(None, gobject.TYPE_PYOBJECT)],              ROW_MODINFO,    False, False))
+
+        self.list = extListview.ExtListView(columns, sortable=False, useMarkup=True, canShowHideColumns=False)
         self.list.set_headers_visible(False)
         self.list.addColumnAttribute(0, toggleRdr, 'activatable', ROW_UNLOADABLE)
         toggleRdr.connect('toggled', self.onModuleToggled)
         self.window.getWidget('scrolledwindow1').add(self.list)
         self.fillList()
+
+        self.list.get_column(1).set_cell_data_func(textRdr, self.__fmtColumnColor)
+
+        # Categories
+        self.iconview = self.window.getWidget('iconview')
+        # Create the store for the iconview
+        self.iconviewStore = gtk.ListStore(gobject.TYPE_STRING, gtk.gdk.Pixbuf, gobject.TYPE_INT)
+        categories = [
+                        (_('Decibel'),  icons.catDecibelIcon(),  consts.MODCAT_DECIBEL),
+                        (_('Desktop'),  icons.catDesktopIcon(),  consts.MODCAT_DESKTOP),
+                        (_('Internet'), icons.catInternetIcon(), consts.MODCAT_INTERNET),
+                        (_('Explorer'), icons.catExplorerIcon(), consts.MODCAT_EXPLORER),
+                     ]
+
+        for category in sorted(categories):
+            self.iconviewStore.append([category[0], category[1], category[2]])
+
+        self.iconview.set_model(self.iconviewStore)
+        self.iconview.set_text_column(0)
+        self.iconview.set_pixbuf_column(1)
+        self.iconview.set_size_request(84, -1)
+
         # GTK handlers
+        self.iconview.connect('selection-changed', self.onCategoryChanged)
         self.window.getWidget('btn-help').connect('clicked', self.onHelp)
+        self.list.connect('extlistview-button-pressed', self.onButtonPressed)
         self.list.get_selection().connect('changed', self.onSelectionChanged)
         self.window.getWidget('btn-prefs').connect('clicked', self.onPreferences)
         self.window.getWidget('btn-close').connect('clicked', lambda btn: self.window.hide())
 
 
+    def __fmtColumnColor(self, col, cll, mdl, it):
+        """ Grey out module that are not enabled """
+        style   = self.list.get_style()
+        enabled = mdl.get_value(it, ROW_ENABLED)
+
+        if enabled: cll.set_property('foreground-gdk', style.text[gtk.STATE_NORMAL])
+        else:       cll.set_property('foreground-gdk', style.text[gtk.STATE_INSENSITIVE])
+
+
     def show(self):
         """ Show the dialog box """
         if not self.window.isVisible():
             self.list.unselectAll()
-            self.window.getWidget('btn-close').grab_focus()
+            self.iconview.grab_focus()
+            self.iconview.select_path((0,))
             self.window.getWidget('btn-prefs').set_sensitive(False)
         self.window.show()
 
 
     def fillList(self):
-        """ Fill the list of modules """
+        """ Fill the list of modules according to the currently selected category """
         rows = []
         for (name, data) in modules.getModules():
             instance     = data[modules.MOD_INSTANCE]
+            category     = data[modules.MOD_INFO][modules.MODINFO_CATEGORY]
             mandatory    = data[modules.MOD_INFO][modules.MODINFO_MANDATORY]
             configurable = data[modules.MOD_INFO][modules.MODINFO_CONFIGURABLE]
 
-            if configurable or not mandatory:
-                if configurable and instance is not None: icon = tools.consts.icoBtnPrefs
+            if (configurable or not mandatory) and category == self.currCat:
+                if configurable and instance is not None: icon = icons.prefsBtnIcon()
                 else:                                     icon = None
 
-                text = '<b>%s</b>\n<small>%s</small>' % (cgi.escape(_(name)), cgi.escape(data[modules.MOD_INFO][modules.MODINFO_DESC]))
+                text = '<b>%s</b>\n<small>%s</small>' % (tools.htmlEscape(_(name)), tools.htmlEscape(data[modules.MOD_INFO][modules.MODINFO_DESC]))
                 rows.append((instance is not None, text, icon, not mandatory, instance, data[modules.MOD_INFO]))
 
         rows.sort(key=lambda row: row[ROW_TEXT])
@@ -90,6 +134,15 @@ class Preferences:
     # --== GTK handlers ==--
 
 
+    def onButtonPressed(self, list, event, path):
+        """ A mouse button has been pressed """
+        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS and path is not None:
+            # Double-clicking an enabled and configurable module opens the configuration dialog
+            row = self.list.getRow(path)
+            if row[ROW_ENABLED] and row[ROW_ICON] is not None:
+                row[ROW_INSTANCE].configure(self.window)
+
+
     def onModuleToggled(self, renderer, path):
         """ A module has been enabled/disabled """
         row  = self.list.getRow(path)
@@ -104,9 +157,21 @@ class Preferences:
         self.fillList()
 
 
+    def onCategoryChanged(self, iconview):
+        """ A new category has been selected """
+        selection = iconview.get_selected_items()
+
+        if len(selection) == 0: self.currCat = consts.MODCAT_NONE
+        else:                   self.currCat = self.iconviewStore[selection[0][0]][2]
+
+        self.fillList()
+
+
     def onHelp(self, btn):
         """ Show a small help message box """
-        helpDlg = gui.help.HelpDlg(_('Modules'))
+        from gui import help
+
+        helpDlg = help.HelpDlg(_('Modules'))
         helpDlg.addSection(_('Description'),
                            _('This dialog box shows the list of available modules, which are small pieces of code that add '
                              'some functionnalities to the application. You can enable/disable a module by checking/unchecking '
