@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import cgi, urllib
+import os.path, tools
 
 from tools   import consts, sec2str
 from gettext import gettext as _
@@ -143,7 +143,7 @@ class Track:
 
         if bitrate == -1:            return _('N/A')
         elif self.getEncMode() == 1: return '~ %u kbps' % (bitrate / 1000)
-        else:                        return '= %u kbps' % (bitrate / 1000)
+        else:                        return '%u kbps'   % (bitrate / 1000)
 
     def getSampleRate(self):
         """ Transform the sample rate into a string"""
@@ -172,6 +172,12 @@ class Track:
             return self.getAlbum()
 
 
+    def getType(self):
+        """ Return the format of the track """
+        if self.tags[TAG_SCH] == 'cdda': return _('CDDA Track')
+        else:                            return os.path.splitext(self.tags[TAG_RES])[1][1:].lower()
+
+
     def __str__(self):
         """ String representation """
         return '%s - %s - %s (%u)' % (self.getArtist(), self.getAlbum(), self.getTitle(), self.getNumber())
@@ -239,24 +245,53 @@ class Track:
         """
         result = fmtString
 
-        result = result.replace( '{path}',         cgi.escape(self.getFilePath()) )
-        result = result.replace( '{album}',        cgi.escape(self.getAlbum())    )
-        result = result.replace( '{track}',        str(self.getNumber())          )
-        result = result.replace( '{title}',        cgi.escape(self.getTitle())    )
-        result = result.replace( '{artist}',       cgi.escape(self.getArtist())   )
-        result = result.replace( '{genre}',        cgi.escape(self.getGenre())    )
-        result = result.replace( '{date}',         str(self.getDate())            )
-        result = result.replace( '{disc}',         str(self.getDiscNumber())      )
-        result = result.replace( '{bitrate}',      self.getBitrate()              )
-        result = result.replace( '{sample_rate}',  self.getSampleRate()           )
-        result = result.replace( '{duration_sec}', str(self.getLength())          )
-        result = result.replace( '{duration_str}', sec2str(self.getLength())      )
-        result = result.replace( '{playlist_pos}', str(self.getPlaylistPos())     )
-        result = result.replace( '{playlist_len}', str(self.getPlaylistLen())     )
+        result = result.replace( '{path}',         tools.htmlEscape(self.getFilePath()) )
+        result = result.replace( '{album}',        tools.htmlEscape(self.getAlbum())    )
+        result = result.replace( '{track}',        str(self.getNumber())                )
+        result = result.replace( '{title}',        tools.htmlEscape(self.getTitle())    )
+        result = result.replace( '{artist}',       tools.htmlEscape(self.getArtist())   )
+        result = result.replace( '{genre}',        tools.htmlEscape(self.getGenre())    )
+        result = result.replace( '{date}',         str(self.getDate())                  )
+        result = result.replace( '{disc}',         str(self.getDiscNumber())            )
+        result = result.replace( '{bitrate}',      self.getBitrate()                    )
+        result = result.replace( '{sample_rate}',  self.getSampleRate()                 )
+        result = result.replace( '{duration_sec}', str(self.getLength())                )
+        result = result.replace( '{duration_str}', sec2str(self.getLength())            )
+        result = result.replace( '{playlist_pos}', str(self.getPlaylistPos())           )
+        result = result.replace( '{playlist_len}', str(self.getPlaylistLen())           )
 
         return result
 
 
+    def __addIfKnown(self, dic, key, tag, unknownValue):
+        """ This is an helper function used by the getMPRISMetadata() function  """
+        value = self.__get(tag, unknownValue)
+        if value != unknownValue:
+            dic[key] = value
+
+
+    def getMPRISMetadata(self):
+        """ Return a dictionary with all available data in an MPRIS-compatible format """
+        data = {'location': self.getURI()}
+
+        self.__addIfKnown(data, 'tracknumber',      TAG_NUM, consts.UNKNOWN_TRACK_NUMBER)
+        self.__addIfKnown(data, 'title',            TAG_TIT, consts.UNKNOWN_TITLE)
+        self.__addIfKnown(data, 'time',             TAG_LEN, consts.UNKNOWN_LENGTH)
+        self.__addIfKnown(data, 'artist',           TAG_ART, consts.UNKNOWN_ARTIST)
+        self.__addIfKnown(data, 'album',            TAG_ALB, consts.UNKNOWN_ALBUM)
+        self.__addIfKnown(data, 'mb track id',      TAG_MBT, consts.UNKNOWN_MB_TRACKID)
+        self.__addIfKnown(data, 'genre',            TAG_GEN, consts.UNKNOWN_GENRE)
+        self.__addIfKnown(data, 'date',             TAG_DAT, consts.UNKNOWN_DATE)
+        self.__addIfKnown(data, 'audio-bitrate',    TAG_BTR, -1)
+        self.__addIfKnown(data, 'audio-samplerate', TAG_SMP, consts.UNKNOWN_SAMPLE_RATE)
+
+        # 'mtime' must be in milliseconds
+        if 'time' in data:
+            data['mtime'] = data['time'] * 1000
+
+        return data
+
+
     def getTags(self):
         """ Return the disctionary of tags """
         return self.tags
@@ -272,7 +307,7 @@ class Track:
         tags = []
         for tag, value in self.tags.iteritems():
             tags.append(str(tag))
-            tags.append(urllib.quote(str(value)))
+            tags.append((str(value)).replace(' ', '\x00'))
         return ' '.join(tags)
 
 
@@ -283,7 +318,7 @@ class Track:
             tag = int(tags[i])
 
             if tag in (TAG_NUM, TAG_LEN, TAG_DNB, TAG_DAT, TAG_PLP, TAG_PLL, TAG_BTR, TAG_SMP, TAG_MOD): self.tags[tag] = int(tags[i+1])
-            else:                                                                                        self.tags[tag] = urllib.unquote(tags[i+1])
+            else:                                                                                        self.tags[tag] = tags[i+1].replace('\x00', ' ')
 
 
 def unserialize(serialTrack):
