@@ -16,15 +16,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import cgi, gobject, gtk, gui, modules, os, socket, tools, traceback, urllib
+import gobject, gtk, gui, modules, os, tools, traceback
 
 from gui                 import extTreeview
-from tools               import consts, prefs, sec2str
+from tools               import consts, icons, prefs, sec2str
 from gettext             import gettext as _
 from tools.log           import logger
 from media.track.cdTrack import CDTrack
 
-MOD_INFO = ('Audio CD', _('Audio CD'), _('Play audio discs'), ('DiscID', 'CDDB'), False, True)
+MOD_INFO = ('Audio CD', _('Audio CD'), _('Play audio discs'), ('DiscID', 'CDDB'), False, True, consts.MODCAT_EXPLORER)
 
 MOD_L10N             = MOD_INFO[modules.MODINFO_L10N]
 PREFS_DFT_DEVICE     = '/dev/cdrom'
@@ -74,42 +74,15 @@ class AudioCD(modules.ThreadedModule):
 
     def __init__(self):
         """ Constructor """
-        modules.ThreadedModule.__init__(self, (consts.MSG_EVT_APP_QUIT,    consts.MSG_EVT_MOD_LOADED, consts.MSG_EVT_EXPLORER_CHANGED,
-                                               consts.MSG_EVT_APP_STARTED, consts.MSG_EVT_MOD_UNLOADED))
+        handlers = {
+                        consts.MSG_EVT_APP_QUIT:         self.onModUnloaded,
+                        consts.MSG_EVT_MOD_LOADED:       self.onModLoaded,
+                        consts.MSG_EVT_APP_STARTED:      self.onModLoaded,
+                        consts.MSG_EVT_MOD_UNLOADED:     self.onModUnloaded,
+                        consts.MSG_EVT_EXPLORER_CHANGED: self.onExplorerChanged,
+                   }
 
-
-    def onModLoaded(self):
-        """ The module has been loaded """
-        txtRdrLen = gtk.CellRendererText()
-
-        columns = (('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf), (txtRdrLen, gobject.TYPE_STRING), (gtk.CellRendererText(), gobject.TYPE_STRING)], True),
-                   (None, [(None, gobject.TYPE_PYOBJECT)],                                                                                               False))
-
-        # The album length is written in a smaller font, with a lighter color
-        txtRdrLen.set_property('scale', 0.85)
-        txtRdrLen.set_property('foreground', '#909090')
-
-        self.tree     = extTreeview.ExtTreeView(columns, True)
-        self.popup    = None
-        self.cfgWin   = None
-        self.expName  = MOD_L10N
-        self.scrolled = gtk.ScrolledWindow()
-        self.cacheDir = os.path.join(consts.dirCfg, MOD_INFO[modules.MODINFO_NAME])
-        # Explorer
-        self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_TRACKS]])
-        self.scrolled.add(self.tree)
-        self.scrolled.set_shadow_type(gtk.SHADOW_IN)
-        self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.scrolled.show()
-        # GTK handlers
-        self.tree.connect('drag-data-get',              self.onDragDataGet)
-        self.tree.connect('key-press-event',            self.onKeyPressed)
-        self.tree.connect('exttreeview-button-pressed', self.onButtonPressed)
-        modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': self.expName, 'icon': consts.icoCdrom, 'widget': self.scrolled})
-        # Hide the album length when not drawing the root node
-        self.tree.get_column(0).set_cell_data_func(txtRdrLen, self.__drawAlbumLenCell)
-        # CD-ROM drive read speed
-        modules.postMsg(consts.MSG_CMD_SET_CD_SPEED, {'speed': prefs.get(__name__, 'read-speed', PREFS_DFT_READ_SPEED)})
+        modules.ThreadedModule.__init__(self, handlers)
 
 
     def __drawAlbumLenCell(self, column, cell, model, iter):
@@ -118,13 +91,6 @@ class AudioCD(modules.ThreadedModule):
         else:                                         cell.set_property('visible', True)
 
 
-    def onModUnloaded(self):
-        """ The module is going to be unloaded """
-        modules.postMsg(consts.MSG_CMD_EXPLORER_REMOVE, {'modName': MOD_L10N, 'expName': MOD_L10N})
-        if not prefs.get(__name__, 'use-cache', PREFS_DFT_USE_CACHE):
-            self.clearCache()
-
-
     def getTracksFromPaths(self, tree, paths):
         """
             Return a list of tracks with all the associated tags:
@@ -148,7 +114,7 @@ class AudioCD(modules.ThreadedModule):
             tracks = self.getTracksFromPaths(tree, paths)
 
             if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
-            else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
+            else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks, 'playNow': False})
 
 
     # --== Cache management ==--
@@ -186,10 +152,10 @@ class AudioCD(modules.ThreadedModule):
     def createTree(self, nbTracks):
         """ Create a temporary explorer tree without disc information """
         name = '%s  <span size="smaller" foreground="#909090">%s</span>' % (MOD_L10N, _('downloading data...'))
-        self.tree.replaceContent(((consts.icoCdrom, None, name, None),))
+        self.tree.replaceContent(((icons.cdromMenuIcon(), None, name, None),))
 
         # Append a child for each track
-        self.tree.appendRows([(consts.icoMediaFile, None, _('Track %02u') % (i+1), None) for i in xrange(nbTracks)], (0,))
+        self.tree.appendRows([(icons.mediaFileMenuIcon(), None, _('Track %02u') % (i+1), None) for i in xrange(nbTracks)], (0,))
         self.tree.expand_all()
 
 
@@ -211,7 +177,7 @@ class AudioCD(modules.ThreadedModule):
         disc          = cddb['DTITLE'].strip().decode('iso-8859-15', 'replace')
         artist, album = disc.split(' / ')
 
-        self.tree.setItem((0,), ROW_NAME, '%s' % cgi.escape(disc))
+        self.tree.setItem((0,), ROW_NAME, '%s' % tools.htmlEscape(disc))
         self.tree.setItem((0,), ROW_LENGTH, '[%s]' % sec2str(sum(trackLen)))
 
         # Update the explorer name
@@ -240,7 +206,7 @@ class AudioCD(modules.ThreadedModule):
             if date is not None:  track.setDate(date)
             if genre is not None: track.setGenre(genre)
             # Fill the tree
-            self.tree.setItem(child, ROW_NAME, '%02u. %s' % (i + 1, cgi.escape(title)))
+            self.tree.setItem(child, ROW_NAME, '%02u. %s' % (i + 1, tools.htmlEscape(title)))
             self.tree.setItem(child, ROW_TRACK, track)
 
 
@@ -249,7 +215,7 @@ class AudioCD(modules.ThreadedModule):
 
     def cddbRequest(self, discInfo):
         """ Return disc information from online CDDB, None if request fails """
-        import CDDB
+        import CDDB, socket
 
         # Make sure to not be blocked by the request
         socket.setdefaulttimeout(consts.socketTimeout)
@@ -279,7 +245,7 @@ class AudioCD(modules.ThreadedModule):
             discInfo = DiscID.disc_id(DiscID.open(prefs.get(__name__, 'device', PREFS_DFT_DEVICE)))
         except Exception, err:
             if err[0] == 123:
-                self.tree.replaceContent([(consts.icoCdrom, None, _('No disc found'), None)])
+                self.tree.replaceContent([(icons.cdromMenuIcon(), None, _('No disc found'), None)])
                 modules.postMsg(consts.MSG_CMD_EXPLORER_RENAME, {'modName': MOD_L10N, 'expName': self.expName, 'newExpName': MOD_L10N})
                 self.expName = MOD_L10N
             else:
@@ -297,20 +263,56 @@ class AudioCD(modules.ThreadedModule):
 
     def reloadDisc(self):
         """ Reload the disc """
-        # Sort of hack, to be sure that the reloading is done in the thread's code and not in the GTK main loop
-        self.postMsg(consts.MSG_EVT_EXPLORER_CHANGED, {'modName': MOD_L10N, 'expName': self.expName})
+        # Make sure the reload is done in the thread's code and not in the GTK main loop
+        self.threadExecute(self.loadDisc)
 
 
-    # --== Message handler ==--
+    # --== Message handlers ==--
 
 
-    def handleMsg(self, msg, params):
-        """ Handle messages sent to this module """
-        if msg == consts.MSG_EVT_MOD_LOADED or msg == consts.MSG_EVT_APP_STARTED:
-            self.onModLoaded()
-        elif msg == consts.MSG_EVT_MOD_UNLOADED or msg == consts.MSG_EVT_APP_QUIT:
-            self.onModUnloaded()
-        elif msg == consts.MSG_EVT_EXPLORER_CHANGED and params['modName'] == MOD_L10N:
+    def onModLoaded(self):
+        """ The module has been loaded """
+        txtRdrLen = gtk.CellRendererText()
+
+        columns = (('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf), (txtRdrLen, gobject.TYPE_STRING), (gtk.CellRendererText(), gobject.TYPE_STRING)], True),
+                   (None, [(None, gobject.TYPE_PYOBJECT)],                                                                                               False))
+
+        self.tree     = extTreeview.ExtTreeView(columns, True)
+        self.popup    = None
+        self.cfgWin   = None
+        self.expName  = MOD_L10N
+        self.scrolled = gtk.ScrolledWindow()
+        self.cacheDir = os.path.join(consts.dirCfg, MOD_INFO[modules.MODINFO_NAME])
+        # The album length is written in a smaller font, with a lighter color
+        txtRdrLen.set_property('scale', 0.85)
+        txtRdrLen.set_property('foreground-gdk', self.tree.get_style().text[gtk.STATE_INSENSITIVE])
+        # Explorer
+        self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_TRACKS]])
+        self.scrolled.add(self.tree)
+        self.scrolled.set_shadow_type(gtk.SHADOW_IN)
+        self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.scrolled.show()
+        # GTK handlers
+        self.tree.connect('drag-data-get',              self.onDragDataGet)
+        self.tree.connect('key-press-event',            self.onKeyPressed)
+        self.tree.connect('exttreeview-button-pressed', self.onButtonPressed)
+        modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': self.expName, 'icon': icons.cdromMenuIcon(), 'widget': self.scrolled})
+        # Hide the album length when not drawing the root node
+        self.tree.get_column(0).set_cell_data_func(txtRdrLen, self.__drawAlbumLenCell)
+        # CD-ROM drive read speed
+        modules.postMsg(consts.MSG_CMD_SET_CD_SPEED, {'speed': prefs.get(__name__, 'read-speed', PREFS_DFT_READ_SPEED)})
+
+
+    def onModUnloaded(self):
+        """ The module is going to be unloaded """
+        modules.postMsg(consts.MSG_CMD_EXPLORER_REMOVE, {'modName': MOD_L10N, 'expName': self.expName})
+        if not prefs.get(__name__, 'use-cache', PREFS_DFT_USE_CACHE):
+            self.clearCache()
+
+
+    def onExplorerChanged(self, modName, expName):
+        """ A new explorer has been selected """
+        if modName == MOD_L10N:
             self.loadDisc()
 
 
@@ -325,22 +327,21 @@ class AudioCD(modules.ThreadedModule):
 
     def onShowPopupMenu(self, tree, button, time, path):
         """ Show a popup menu """
-        if self.popup is None:
-            self.popup = tools.loadGladeFile('AudioCDMenu.glade')
-            self.popup.get_widget('menu-popup').show_all()
-            self.popup.get_widget('item-add').connect('activate',     lambda widget: self.playPaths(tree, None, False))
-            self.popup.get_widget('item-play').connect('activate',    lambda widget: self.playPaths(tree, None, True))
-            self.popup.get_widget('item-refresh').connect('activate', lambda widget: self.reloadDisc())
+        popup = gtk.Menu()
 
-        # Enable/disable menu entries depending on whether there is something to play
-        if self.tree.getNbChildren((0,)) == 0:
-            self.popup.get_widget('item-add').set_sensitive(False)
-            self.popup.get_widget('item-play').set_sensitive(False)
-        else:
-            self.popup.get_widget('item-add').set_sensitive(True)
-            self.popup.get_widget('item-play').set_sensitive(True)
+        # Play selection
+        play = gtk.ImageMenuItem(gtk.STOCK_MEDIA_PLAY)
+        play.set_sensitive(tree.getNbChildren((0,)) != 0)
+        play.connect('activate', lambda widget: self.playPaths(tree, None, True))
+        popup.append(play)
 
-        self.popup.get_widget('menu-popup').popup(None, None, None, button, time)
+        # Refresh the view
+        refresh = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
+        refresh.connect('activate', lambda widget: self.reloadDisc())
+        popup.append(refresh)
+
+        popup.show_all()
+        popup.popup(None, None, None, button, time)
 
 
     def onButtonPressed(self, tree, event, path):
