@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import gui, Image, modules, os, socket, tempfile, tools, traceback, urllib2
+import modules, os, tools, traceback
 
 from tools     import consts, prefs
 from gettext   import gettext as _
@@ -24,9 +24,8 @@ from tools.log import logger
 
 
 # Module information
-MOD_INFO = ('Covers', _('Covers'), _('Show album covers'), [], False, True)
+MOD_INFO = ('Covers', _('Covers'), _('Show album covers'), [], False, True, consts.MODCAT_DECIBEL)
 MOD_NAME = MOD_INFO[modules.MODINFO_NAME]
-MOD_L10N = MOD_INFO[modules.MODINFO_L10N]
 
 AS_API_KEY   = 'fd8dd98d26bb3f288f3e626502f9add6'   # Ingelrest François' Audioscrobbler API key
 AS_TAG_START = '<image size="large">'               # The text that is right before the URL to the cover
@@ -38,7 +37,13 @@ USER_AGENT = 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/20080728
 # We store both the paths to the thumbnail and to the full size image
 (
     CVR_THUMB,
-    CVR_FULL
+    CVR_FULL,
+) = range(2)
+
+# Width/height of PIL images
+(
+    PIL_WIDTH,
+    PIL_HEIGHT,
 ) = range(2)
 
 # Constants for thumbnails
@@ -48,16 +53,17 @@ THUMBNAIL_OFFSETX =  11  # X-offset to render the thumbnail in the model
 THUMBNAIL_OFFSETY =   3  # Y-offset to render the thumbnail in the model
 
 # Constants for full size covers
-FULL_SIZE_COVER_WIDTH  = 300
-FULL_SIZE_COVER_HEIGHT = 300
+FULLSIZE_WIDTH  = 300
+FULLSIZE_HEIGHT = 300
 
 # File formats we can read
 ACCEPTED_FILE_FORMATS = {'.jpg': None, '.jpeg': None, '.png': None, '.gif': None}
 
 # Default preferences
-PREFS_DFT_DOWNLOAD_COVERS      = False
-PREFS_DFT_PREFER_USER_COVERS   = True
-PREFS_DFT_USER_COVER_FILENAMES = ['cover', 'art', 'front', '*']
+PREFS_DFT_DOWNLOAD_COVERS       = False
+PREFS_DFT_PREFER_USER_COVERS    = True
+PREFS_DFT_USER_COVER_FILENAMES  = ['cover', 'art', 'front', '*']
+PREFS_DFT_SEARCH_IN_PARENT_DIRS = False
 
 # Images for thumbnails
 THUMBNAIL_GLOSS = os.path.join(consts.dirPix, 'cover-gloss.png')
@@ -68,52 +74,51 @@ class Covers(modules.ThreadedModule):
 
     def __init__(self):
         """ Constructor """
-        modules.ThreadedModule.__init__(self, (consts.MSG_EVT_MOD_LOADED,   consts.MSG_EVT_APP_STARTED, consts.MSG_EVT_NEW_TRACK,
-                                               consts.MSG_EVT_MOD_UNLOADED, consts.MSG_EVT_APP_QUIT))
+        handlers = {
+                        consts.MSG_EVT_APP_QUIT:     self.onModUnloaded,
+                        consts.MSG_EVT_NEW_TRACK:    self.onNewTrack,
+                        consts.MSG_EVT_MOD_LOADED:   self.onModLoaded,
+                        consts.MSG_EVT_APP_STARTED:  self.onModLoaded,
+                        consts.MSG_EVT_MOD_UNLOADED: self.onModUnloaded,
+                   }
+
+        modules.ThreadedModule.__init__(self, handlers)
 
 
-    def onModLoaded(self):
-        """ The module has been loaded """
-        self.cfgWin         = None                                   # Configuration window
-        self.coverMap       = {}                                     # Store covers previously requested
-        self.currTrack      = None                                   # The current track being played, if any
-        self.cacheRootPath  = os.path.join(consts.dirCfg, MOD_NAME)  # Local cache for Internet covers
-        self.coverBlacklist = {}                                     # When a cover cannot be downloaded, avoid requesting it again
+    def __resizeWithRatio(self, width, height, maxWidth, maxHeight):
+        """
+            Fit (width x height) into (maxWidth x maxHeight) while preserving the original ratio
 
-        if not os.path.exists(self.cacheRootPath):
-            os.mkdir(self.cacheRootPath)
+            Return a tuple (newWidth, newheight)
+        """
+        diffWidth  = width - maxWidth
+        diffHeight = height - maxHeight
 
+        if diffWidth <= 0 and diffHeight <= 0:
+            newWidth  = width
+            newHeight = height
+        elif diffHeight > diffWidth:
+            newHeight = maxHeight
+            newWidth  = width * maxHeight / height
+        else:
+            newWidth  = maxWidth
+            newHeight = height * maxWidth / width
 
-    def onModUnloaded(self):
-        """ The module has been unloaded """
-        if self.currTrack is not None:
-            modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': self.currTrack, 'pathThumbnail': None, 'pathFullSize': None})
-
-        # Delete covers that have been generated by this module
-        for covers in self.coverMap.itervalues():
-            if os.path.exists(covers[CVR_THUMB]):
-                os.remove(covers[CVR_THUMB])
-            if os.path.exists(covers[CVR_FULL]):
-                os.remove(covers[CVR_FULL])
-        self.coverMap = None
-
-        # Delete blacklist
-        self.coverBlacklist = None
+        return (newWidth, newHeight)
 
 
     def generateFullSizeCover(self, inFile, outFile, format):
         """ Resize inFile if needed, and write it to outFile (outFile and inFile may be equal) """
+        import Image
+
         try:
             # Open the image
             cover = Image.open(inFile)
 
-            # Resize in the best way we can
-            if cover.size[0] < FULL_SIZE_COVER_WIDTH: newWidth = cover.size[0]
-            else:                                     newWidth = FULL_SIZE_COVER_WIDTH
-
-            if cover.size[1] < FULL_SIZE_COVER_HEIGHT: newHeight = cover.size[1]
-            else:                                      newHeight = FULL_SIZE_COVER_HEIGHT
+            # Fit the image into FULLSIZE_WIDTH x FULLSIZE_HEIGHT
+            (newWidth, newHeight) = self.__resizeWithRatio(cover.size[PIL_WIDTH], cover.size[PIL_HEIGHT], FULLSIZE_WIDTH, FULLSIZE_HEIGHT)
 
+            # Resize it
             cover = cover.resize((newWidth, newHeight), Image.ANTIALIAS)
 
             # We're done
@@ -124,25 +129,23 @@ class Covers(modules.ThreadedModule):
 
     def generateThumbnail(self, inFile, outFile, format):
         """ Generate a thumbnail from inFile (e.g., resize it) and write it to outFile (outFile and inFile may be equal) """
+        import Image
+
         try:
             # Open the image
             cover = Image.open(inFile).convert('RGBA')
 
-            # Resize in the best way we can
-            if cover.size[0] < THUMBNAIL_WIDTH:
-                newWidth = cover.size[0]
-                offsetX  = (THUMBNAIL_WIDTH - cover.size[0]) / 2
-            else:
-                newWidth = THUMBNAIL_WIDTH
-                offsetX  = 0
+            # Fit the image into THUMBNAIL_WIDTH x THUMBNAIL_HEIGHT
+            (newWidth, newHeight) = self.__resizeWithRatio(cover.size[PIL_WIDTH], cover.size[PIL_HEIGHT], THUMBNAIL_WIDTH, THUMBNAIL_HEIGHT)
 
-            if cover.size[1] < THUMBNAIL_HEIGHT:
-                newHeight = cover.size[1]
-                offsetY   = (THUMBNAIL_HEIGHT - cover.size[1]) / 2
-            else:
-                newHeight = THUMBNAIL_HEIGHT
-                offsetY   = 0
+            # We need to shift the image if it doesn't fully fill the thumbnail
+            if newWidth < THUMBNAIL_WIDTH: offsetX = (THUMBNAIL_WIDTH - newWidth) / 2
+            else:                          offsetX = 0
 
+            if newHeight < THUMBNAIL_HEIGHT: offsetY = (THUMBNAIL_HEIGHT - newHeight) / 2
+            else:                            offsetY = 0
+
+            # Resize the image
             cover = cover.resize((newWidth, newHeight), Image.ANTIALIAS)
 
             # Paste the resized cover into our model
@@ -163,20 +166,34 @@ class Covers(modules.ThreadedModule):
 
     def getUserCover(self, trackPath):
         """ Return the path to a cover file in trackPath, None if no cover found """
-        # Create a dictionary with candidates
-        candidates = {}
-        for (file, path) in tools.listDir(trackPath, True):
-            (name, ext) = os.path.splitext(file.lower())
-            if ext in ACCEPTED_FILE_FORMATS:
-                candidates[name] = path
+        splitPath = tools.splitPath(trackPath)
 
-        # Check each possible name using the its index in the list as its priority
-        for name in prefs.get(__name__, 'user-cover-filenames', PREFS_DFT_USER_COVER_FILENAMES):
-            if name in candidates:
-                return candidates[name]
+        if prefs.get(__name__, 'search-in-parent-dirs', PREFS_DFT_SEARCH_IN_PARENT_DIRS): lvls = len(splitPath)
+        else:                                                                             lvls = 1
 
-            if name == '*' and len(candidates) != 0:
-                return candidates.values()[0]
+        while lvls != 0:
+
+            # Create the path we're currently looking into
+            currPath = os.path.join(*splitPath)
+
+            # Create a dictionary with candidates
+            candidates = {}
+            for (file, path) in tools.listDir(currPath, True):
+                (name, ext) = os.path.splitext(file.lower())
+                if ext in ACCEPTED_FILE_FORMATS:
+                    candidates[name] = path
+
+            # Check each possible name using its index in the list as its priority
+            for name in prefs.get(__name__, 'user-cover-filenames', PREFS_DFT_USER_COVER_FILENAMES):
+                if name in candidates:
+                    return candidates[name]
+
+                if name == '*' and len(candidates) != 0:
+                    return candidates.values()[0]
+
+            # No cover found, let's go one level higher
+            lvls      -= 1
+            splitPath  = splitPath[:-1]
 
         return None
 
@@ -203,6 +220,8 @@ class Covers(modules.ThreadedModule):
             If successful, add it to the cache and return the path to it
             Otherwise, return None
         """
+        import socket, urllib2
+
         # Make sure to not be blocked by the request
         socket.setdefaulttimeout(consts.socketTimeout)
 
@@ -280,10 +299,8 @@ class Covers(modules.ThreadedModule):
 
     def getFromInternet(self, artist, album):
         """ Wrapper for __getFromInternet(), manage blacklist """
-        coverKey = artist + album
-
         # If we already tried without success, don't try again
-        if coverKey in self.coverBlacklist:
+        if (artist, album) in self.coverBlacklist:
             return None
 
         # Otherwise, try to download the cover
@@ -291,11 +308,43 @@ class Covers(modules.ThreadedModule):
 
         # If the download failed, blacklist the album
         if cover is None:
-            self.coverBlacklist[coverKey] = None
+            self.coverBlacklist[(artist, album)] = None
 
         return cover
 
 
+    # --== Message handlers ==--
+
+
+    def onModLoaded(self):
+        """ The module has been loaded """
+        self.cfgWin         = None                                   # Configuration window
+        self.coverMap       = {}                                     # Store covers previously requested
+        self.currTrack      = None                                   # The current track being played, if any
+        self.cacheRootPath  = os.path.join(consts.dirCfg, MOD_NAME)  # Local cache for Internet covers
+        self.coverBlacklist = {}                                     # When a cover cannot be downloaded, avoid requesting it again
+
+        if not os.path.exists(self.cacheRootPath):
+            os.mkdir(self.cacheRootPath)
+
+
+    def onModUnloaded(self):
+        """ The module has been unloaded """
+        if self.currTrack is not None:
+            modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': self.currTrack, 'pathThumbnail': None, 'pathFullSize': None})
+
+        # Delete covers that have been generated by this module
+        for covers in self.coverMap.itervalues():
+            if os.path.exists(covers[CVR_THUMB]):
+                os.remove(covers[CVR_THUMB])
+            if os.path.exists(covers[CVR_FULL]):
+                os.remove(covers[CVR_FULL])
+        self.coverMap = None
+
+        # Delete blacklist
+        self.coverBlacklist = None
+
+
     def onNewTrack(self, track):
         """ A new track is being played, try to retrieve the corresponding cover """
         # Make sure we have enough information
@@ -305,13 +354,12 @@ class Covers(modules.ThreadedModule):
 
         album          = track.getAlbum().lower()
         artist         = track.getArtist().lower()
-        coverKey       = artist + album
         rawCover       = None
         self.currTrack = track
 
         # Let's see whether we already have the cover
-        if coverKey in self.coverMap:
-            covers        = self.coverMap[coverKey]
+        if (artist, album) in self.coverMap:
+            covers        = self.coverMap[(artist, album)]
             pathFullSize  = covers[CVR_FULL]
             pathThumbnail = covers[CVR_THUMB]
 
@@ -339,37 +387,28 @@ class Covers(modules.ThreadedModule):
         # If we still don't have a cover, too bad
         # Otherwise, generate a thumbnail and a full size cover, and add it to our cover map
         if rawCover is not None:
+            import tempfile
+
             thumbnail     = tempfile.mktemp() + '.png'
             fullSizeCover = tempfile.mktemp() + '.png'
             self.generateThumbnail(rawCover, thumbnail, 'PNG')
             self.generateFullSizeCover(rawCover, fullSizeCover, 'PNG')
             if os.path.exists(thumbnail) and os.path.exists(fullSizeCover):
-                self.coverMap[coverKey] = (thumbnail, fullSizeCover)
+                self.coverMap[(artist, album)] = (thumbnail, fullSizeCover)
                 modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'pathThumbnail': thumbnail, 'pathFullSize': fullSizeCover})
             else:
                 modules.postMsg(consts.MSG_CMD_SET_COVER, {'track': track, 'pathThumbnail': None, 'pathFullSize': None})
 
 
-    # --== Message handler ==--
-
-
-    def handleMsg(self, msg, params):
-        """ Handle messages sent to this module """
-        if msg == consts.MSG_EVT_NEW_TRACK:
-            self.onNewTrack(params['track'])
-        elif msg in (consts.MSG_EVT_MOD_LOADED, consts.MSG_EVT_APP_STARTED):
-            self.onModLoaded()
-        elif msg in (consts.MSG_EVT_MOD_UNLOADED, consts.MSG_EVT_APP_QUIT):
-            self.onModUnloaded()
-
-
     # --== Configuration ==--
 
 
     def configure(self, parent):
         """ Show the configuration window """
         if self.cfgWin is None:
-            self.cfgWin = gui.window.Window('Covers.glade', 'vbox1', __name__, MOD_L10N, 320, 265)
+            from gui.window import Window
+
+            self.cfgWin = Window('Covers.glade', 'vbox1', __name__, MOD_INFO[modules.MODINFO_L10N], 320, 265)
             self.cfgWin.getWidget('btn-ok').connect('clicked', self.onBtnOk)
             self.cfgWin.getWidget('img-lastfm').set_from_file(os.path.join(consts.dirPix, 'audioscrobbler.png'))
             self.cfgWin.getWidget('btn-help').connect('clicked', self.onBtnHelp)
@@ -377,15 +416,17 @@ class Covers(modules.ThreadedModule):
             self.cfgWin.getWidget('btn-cancel').connect('clicked', lambda btn: self.cfgWin.hide())
 
         if not self.cfgWin.isVisible():
-            downloadCovers     = prefs.get(__name__, 'download-covers',      PREFS_DFT_DOWNLOAD_COVERS)
-            preferUserCovers   = prefs.get(__name__, 'prefer-user-covers',   PREFS_DFT_PREFER_USER_COVERS)
-            userCoverFilenames = prefs.get(__name__, 'user-cover-filenames', PREFS_DFT_USER_COVER_FILENAMES)
+            downloadCovers     = prefs.get(__name__, 'download-covers',       PREFS_DFT_DOWNLOAD_COVERS)
+            preferUserCovers   = prefs.get(__name__, 'prefer-user-covers',    PREFS_DFT_PREFER_USER_COVERS)
+            userCoverFilenames = prefs.get(__name__, 'user-cover-filenames',  PREFS_DFT_USER_COVER_FILENAMES)
+            searchInParentDirs = prefs.get(__name__, 'search-in-parent-dirs', PREFS_DFT_SEARCH_IN_PARENT_DIRS)
 
             self.cfgWin.getWidget('btn-ok').grab_focus()
             self.cfgWin.getWidget('txt-filenames').set_text(', '.join(userCoverFilenames))
             self.cfgWin.getWidget('chk-downloadCovers').set_active(downloadCovers)
             self.cfgWin.getWidget('chk-preferUserCovers').set_active(preferUserCovers)
             self.cfgWin.getWidget('chk-preferUserCovers').set_sensitive(downloadCovers)
+            self.cfgWin.getWidget('chk-searchInParentDirs').set_active(searchInParentDirs)
 
         self.cfgWin.show()
 
@@ -394,11 +435,13 @@ class Covers(modules.ThreadedModule):
         """ Save configuration """
         downloadCovers     = self.cfgWin.getWidget('chk-downloadCovers').get_active()
         preferUserCovers   = self.cfgWin.getWidget('chk-preferUserCovers').get_active()
+        searchInParentDirs = self.cfgWin.getWidget('chk-searchInParentDirs').get_active()
         userCoverFilenames = [word.strip() for word in self.cfgWin.getWidget('txt-filenames').get_text().split(',')]
 
-        prefs.set(__name__, 'download-covers',      downloadCovers)
-        prefs.set(__name__, 'prefer-user-covers',   preferUserCovers)
-        prefs.set(__name__, 'user-cover-filenames', userCoverFilenames)
+        prefs.set(__name__, 'download-covers',       downloadCovers)
+        prefs.set(__name__, 'prefer-user-covers',    preferUserCovers)
+        prefs.set(__name__, 'user-cover-filenames',  userCoverFilenames)
+        prefs.set(__name__, 'search-in-parent-dirs', searchInParentDirs)
 
         self.cfgWin.hide()
 
@@ -410,7 +453,9 @@ class Covers(modules.ThreadedModule):
 
     def onBtnHelp(self, btn):
         """ Display a small help message box """
-        helpDlg = gui.help.HelpDlg(MOD_L10N)
+        from gui import help
+
+        helpDlg = help.HelpDlg(MOD_INFO[modules.MODINFO_L10N])
         helpDlg.addSection(_('Description'),
                            _('This module displays the cover of the album the current track comes from. Covers '
                               'may be loaded from local pictures, located in the same directory as the current '
@@ -418,7 +463,8 @@ class Covers(modules.ThreadedModule):
         helpDlg.addSection(_('User Covers'),
                            _('A user cover is a picture located in the same directory as the current track. '
                              'When specifying filenames, you do not need to provide file extensions, supported '
-                             'file formats (%s) are automatically used.' % ', '.join(ACCEPTED_FILE_FORMATS.iterkeys())))
+                             'file formats (%s) are automatically used. This module can be configured to search '
+                             'for user covers in parent directories are well.' % ', '.join(ACCEPTED_FILE_FORMATS.iterkeys())))
         helpDlg.addSection(_('Internet Covers'),
                            _('Covers may be downloaded from the Internet, based on the tags of the current track. '
                              'You can ask to always prefer user covers to Internet ones. In this case, if a user '
