@@ -16,16 +16,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import cgi, gtk, gui.window, media, modules, os, tools, urllib
+import gtk, media, modules, os, tools
 
-from gui     import fileChooser, help, extTreeview, extListview, selectPath
-from tools   import consts, prefs
+from tools   import consts, prefs, icons
 from media   import playlist
 from gettext import gettext as _
 from os.path import isdir, isfile
 from gobject import idle_add, TYPE_STRING, TYPE_INT
 
-MOD_INFO = ('File Explorer', _('File Explorer'), _('Browse your file system'), [], True, True)
+MOD_INFO = ('File Explorer', _('File Explorer'), _('Browse your file system'), [], True, True, consts.MODCAT_EXPLORER)
 MOD_L10N = MOD_INFO[modules.MODINFO_L10N]
 
 # Default preferences
@@ -45,9 +44,9 @@ PREFS_DEFAULT_SHOW_HIDDEN_FILES = False
 
 # The possible types for a node of the tree
 (
-    TYPE_DIR,   # A directory
-    TYPE_FILE,  # A media file
-    TYPE_NONE   # A fake item, used to display a '+' in front of a directory when needed
+    TYPE_DIR,      # A directory
+    TYPE_FILE,     # A media file
+    TYPE_NONE      # A fake item, used to display a '+' in front of a directory when needed
 ) = range(3)
 
 
@@ -56,46 +55,35 @@ class FileExplorer(modules.Module):
 
     def __init__(self):
         """ Constructor """
-        modules.Module.__init__(self, (consts.MSG_EVT_APP_STARTED, consts.MSG_EVT_EXPLORER_CHANGED, consts.MSG_EVT_APP_QUIT))
+        handlers = {
+                        consts.MSG_EVT_APP_QUIT:         self.onAppQuit,
+                        consts.MSG_EVT_APP_STARTED:      self.onAppStarted,
+                        consts.MSG_EVT_EXPLORER_CHANGED: self.onExplorerChanged,
+                   }
+
+        modules.Module.__init__(self, handlers)
 
 
-    def onModLoaded(self):
-        """ The module has been loaded """
-        self.tree            = None
-        self.cfgWin          = None
-        self.folders         = prefs.get(__name__, 'media-folders', PREFS_DEFAULT_MEDIA_FOLDERS)
-        self.scrolled        = gtk.ScrolledWindow()
-        self.currRoot        = None
-        self.addByFilename   = prefs.get(__name__, 'add-by-filename',  PREFS_DEFAULT_ADD_BY_FILENAME)
-        self.showHiddenFiles = prefs.get(__name__, 'show-hidden-files', PREFS_DEFAULT_SHOW_HIDDEN_FILES)
+    def createTree(self):
+        """ Create the tree used to display the file system """
+        from gui import extTreeview
 
-        self.scrolled.set_shadow_type(gtk.SHADOW_IN)
-        self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.scrolled.show()
+        columns = (('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf), (gtk.CellRendererText(), TYPE_STRING)], True),
+                   (None, [(None, TYPE_INT)],                                                                  False),
+                   (None, [(None, TYPE_STRING)],                                                               False))
 
-        for name in self.folders:
-            modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': name, 'icon': None, 'widget': self.scrolled})
+        self.tree = extTreeview.ExtTreeView(columns, True)
+
+        self.scrolled.add(self.tree)
+        self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_TRACKS]])
+        self.tree.connect('drag-data-get', self.onDragDataGet)
+        self.tree.connect('key-press-event', self.onKeyPressed)
+        self.tree.connect('exttreeview-button-pressed', self.onMouseButton)
+        self.tree.connect('exttreeview-row-collapsed', self.onRowCollapsed)
+        self.tree.connect('exttreeview-row-expanded', self.onRowExpanded)
 
 
-    def onModUnloaded(self):
-        """ The module is going to be unloaded """
-        prefs.set(__name__, 'media-folders',     self.folders)
-        prefs.set(__name__, 'add-by-filename',   self.addByFilename)
-        prefs.set(__name__, 'show-hidden-files', self.showHiddenFiles)
-
-        # Save the state of the current tree if needed
-        if self.currRoot is not None:
-            savedStates = prefs.get(__name__, 'saved-states', {})
-            savedStates[self.currRoot] = {
-                                            'tree-state':     self.dumpTree(),
-                                            'selected-paths': self.tree.getSelectedPaths(),
-                                            'vscrollbar-pos': self.scrolled.get_vscrollbar().get_value(),
-                                            'hscrollbar-pos': self.scrolled.get_hscrollbar().get_value(),
-                                         }
-            prefs.set(__name__, 'saved-states', savedStates)
-
-
-    def dumpTree(self, path=None):
+    def getTreeDump(self, path=None):
         """ Recursively dump the given tree starting at path (None for the root of the tree) """
         list = []
 
@@ -103,7 +91,7 @@ class FileExplorer(modules.Module):
             row = self.tree.getRow(child)
 
             if self.tree.getNbChildren(child) == 0: grandChildren = None
-            elif self.tree.row_expanded(child):     grandChildren = self.dumpTree(child)
+            elif self.tree.row_expanded(child):     grandChildren = self.getTreeDump(child)
             else:                                   grandChildren = []
 
             list.append([(row[ROW_NAME], row[ROW_TYPE], row[ROW_FULLPATH]), grandChildren])
@@ -111,26 +99,37 @@ class FileExplorer(modules.Module):
         return list
 
 
-    def restoreTree(self, dump, parent=None):
+    def restoreTreeDump(self, dump, parent=None):
         """ Recursively restore the dump under the given parent (None for the root of the tree) """
         for item in dump:
             (name, type, path) = item[0]
 
             if type == TYPE_FILE:
-                self.tree.appendRow((consts.icoMediaFile, name, TYPE_FILE, path), parent)
+                self.tree.appendRow((icons.mediaFileMenuIcon(), name, TYPE_FILE, path), parent)
             else:
-                newNode = self.tree.appendRow((consts.icoDir, name, TYPE_DIR, path), parent)
+                newNode = self.tree.appendRow((icons.dirMenuIcon(), name, TYPE_DIR, path), parent)
 
                 if item[1] is not None:
-                    fakeChild = self.tree.appendRow((consts.icoDir, '', TYPE_NONE, ''), newNode)
+                    fakeChild = self.tree.appendRow((icons.dirMenuIcon(), '', TYPE_NONE, ''), newNode)
 
                     if len(item[1]) != 0:
                         # We must expand the row before adding the real children, but this works only if there is already at least one child
                         self.tree.expandRow(newNode)
-                        self.restoreTree(item[1], newNode)
+                        self.restoreTreeDump(item[1], newNode)
                         self.tree.removeRow(fakeChild)
 
 
+    def saveTreeState(self):
+        """ Return a dictionary representing the current state of the tree """
+        if self.currRoot is not None:
+            self.treeState[self.currRoot] = {
+                        'tree-state':     self.getTreeDump(),
+                        'selected-paths': self.tree.getSelectedPaths(),
+                        'vscrollbar-pos': self.scrolled.get_vscrollbar().get_value(),
+                        'hscrollbar-pos': self.scrolled.get_hscrollbar().get_value(),
+                   }
+
+
     def sortKey(self, row):
         """ Key function used to compare two rows of the tree """
         return row[ROW_NAME].lower()
@@ -152,11 +151,11 @@ class FileExplorer(modules.Module):
             Replace/extend the tracklist
             If 'path' is None, use the current selection
         """
-        if path is None: tracks = media.getTracks([row[ROW_FULLPATH] for row in self.tree.getSelectedRows()], self.addByFilename)
-        else:            tracks = media.getTracks([self.tree.getRow(path)[ROW_FULLPATH]], self.addByFilename)
+        if path is None: tracks = media.getTracks([row[ROW_FULLPATH] for row in self.tree.getSelectedRows()], self.addByFilename, not self.showHiddenFiles)
+        else:            tracks = media.getTracks([self.tree.getRow(path)[ROW_FULLPATH]], self.addByFilename, not self.showHiddenFiles)
 
         if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
-        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
+        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks, 'playNow': False})
 
 
     def renameFolder(self, oldName, newName):
@@ -164,13 +163,11 @@ class FileExplorer(modules.Module):
         self.folders[newName] = self.folders[oldName]
         del self.folders[oldName]
 
-        savedStates = prefs.get(__name__, 'saved-states', {})
-        if oldName in savedStates:
-            savedStates[newName] = savedStates[oldName]
-            del savedStates[oldName]
-            prefs.set(__name__, 'saved-states', savedStates)
+        if oldName in self.treeState:
+            self.treeState[newName] = self.treeState[oldName]
+            del self.treeState[oldName]
 
-        modules.postMsg(consts.MSG_CMD_EXPLORER_RENAME,   {'modName': MOD_L10N, 'expName': oldName, 'newExpName': newName})
+        modules.postMsg(consts.MSG_CMD_EXPLORER_RENAME, {'modName': MOD_L10N, 'expName': oldName, 'newExpName': newName})
 
 
     # --== Tree management ==--
@@ -199,12 +196,12 @@ class FileExplorer(modules.Module):
 
         for (file, path) in tools.listDir(directory, self.showHiddenFiles):
             if isdir(path):
-                directories.append((consts.icoDir, cgi.escape(unicode(file, errors='replace')), TYPE_DIR, path))
+                directories.append((icons.dirMenuIcon(), tools.htmlEscape(unicode(file, errors='replace')), TYPE_DIR, path))
             elif isfile(path):
                 if media.isSupported(file):
-                    mediaFiles.append((consts.icoMediaFile, cgi.escape(unicode(file, errors='replace')), TYPE_FILE, path))
+                    mediaFiles.append((icons.mediaFileMenuIcon(), tools.htmlEscape(unicode(file, errors='replace')), TYPE_FILE, path))
                 elif playlist.isSupported(file):
-                    playlists.append((consts.icoMediaFile, cgi.escape(unicode(file, errors='replace')), TYPE_FILE, path))
+                    playlists.append((icons.mediaFileMenuIcon(), tools.htmlEscape(unicode(file, errors='replace')), TYPE_FILE, path))
 
         playlists.sort(key=self.sortKey)
         mediaFiles.sort(key=self.sortKey)
@@ -247,7 +244,7 @@ class FileExplorer(modules.Module):
                         break
 
             # Append/remove children if needed
-            if hasContent and self.tree.getNbChildren(child) == 0:      self.tree.appendRow((consts.icoDir, '', TYPE_NONE, ''), child)
+            if hasContent and self.tree.getNbChildren(child) == 0:      self.tree.appendRow((icons.dirMenuIcon(), '', TYPE_NONE, ''), child)
             elif not hasContent and self.tree.getNbChildren(child) > 0: self.tree.removeAllChildren(child)
 
             yield True
@@ -328,9 +325,9 @@ class FileExplorer(modules.Module):
             if event.button == 2:
                 self.play(False, path)
             elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
-                if   tree.getItem(path, ROW_PIXBUF) != consts.icoDir: self.play(True)
-                elif tree.row_expanded(path):                         tree.collapse_row(path)
-                else:                                                 tree.expand_row(path, False)
+                if   tree.getItem(path, ROW_PIXBUF) != icons.dirMenuIcon(): self.play(True)
+                elif tree.row_expanded(path):                               tree.collapse_row(path)
+                else:                                                       tree.expand_row(path, False)
 
 
     def onShowPopupMenu(self, tree, button, time, path):
@@ -339,15 +336,10 @@ class FileExplorer(modules.Module):
 
         # Play selection
         play = gtk.ImageMenuItem(gtk.STOCK_MEDIA_PLAY)
-        play.set_sensitive(path is not None)
-        play.connect('activate', lambda widget: self.play(True))
         popup.append(play)
 
-        # Add selection
-        add = gtk.ImageMenuItem(gtk.STOCK_ADD)
-        add.set_sensitive(path is not None)
-        add.connect('activate', lambda widget: self.play(False))
-        popup.append(add)
+        if path is None: play.set_sensitive(False)
+        else:            play.connect('activate', lambda widget: self.play(True))
 
         popup.append(gtk.SeparatorMenuItem())
 
@@ -357,7 +349,7 @@ class FileExplorer(modules.Module):
         collapse.connect('activate', lambda widget: tree.collapse_all())
         popup.append(collapse)
 
-        # Refresh the library
+        # Refresh the view
         refresh = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
         refresh.connect('activate', lambda widget: self.refresh())
         popup.append(refresh)
@@ -396,73 +388,74 @@ class FileExplorer(modules.Module):
     def onRowCollapsed(self, tree, path):
         """ Replace all children by a fake child """
         tree.removeAllChildren(path)
-        tree.appendRow((consts.icoDir, '', TYPE_NONE, ''), path)
+        tree.appendRow((icons.dirMenuIcon(), '', TYPE_NONE, ''), path)
 
 
     def onDragDataGet(self, tree, context, selection, info, time):
         """ Provide information about the data being dragged """
-        selection.set('text/uri-list', 8, ' '.join([urllib.pathname2url(file) for file in [row[ROW_FULLPATH] for row in tree.getSelectedRows()]]))
+        allTracks = media.getTracks([row[ROW_FULLPATH] for row in self.tree.getSelectedRows()], self.addByFilename, not self.showHiddenFiles)
+        selection.set(consts.DND_TARGETS[consts.DND_DAP_TRACKS][0], 8, '\n'.join([track.serialize() for track in allTracks]))
 
 
-   # --== Message handler ==--
+   # --== Message handlers ==--
 
 
-    def handleMsg(self, msg, params):
-        """ Handle messages sent to this module """
-        if msg == consts.MSG_EVT_EXPLORER_CHANGED and params['modName'] == MOD_L10N and self.currRoot != params['expName']:
-            newRoot = params['expName']
+    def onAppStarted(self):
+        """ The module has been loaded """
+        self.tree            = None
+        self.cfgWin          = None
+        self.folders         = prefs.get(__name__, 'media-folders', PREFS_DEFAULT_MEDIA_FOLDERS)
+        self.scrolled        = gtk.ScrolledWindow()
+        self.currRoot        = None
+        self.treeState       = prefs.get(__name__, 'saved-states', {})
+        self.addByFilename   = prefs.get(__name__, 'add-by-filename',  PREFS_DEFAULT_ADD_BY_FILENAME)
+        self.showHiddenFiles = prefs.get(__name__, 'show-hidden-files', PREFS_DEFAULT_SHOW_HIDDEN_FILES)
 
-            # Create the tree if needed (this is done only the very first time)
+        self.scrolled.set_shadow_type(gtk.SHADOW_IN)
+        self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.scrolled.show()
+
+        for name in self.folders:
+            modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': name, 'icon': icons.dirMenuIcon(), 'widget': self.scrolled})
+
+
+    def onAppQuit(self):
+        """ The module is going to be unloaded """
+        self.saveTreeState()
+        prefs.set(__name__, 'saved-states',      self.treeState)
+        prefs.set(__name__, 'media-folders',     self.folders)
+        prefs.set(__name__, 'add-by-filename',   self.addByFilename)
+        prefs.set(__name__, 'show-hidden-files', self.showHiddenFiles)
+
+
+    def onExplorerChanged(self, modName, expName):
+        """ A new explorer has been selected """
+        if modName == MOD_L10N and self.currRoot != expName:
+            # Create the tree if needed
             if self.tree is None:
-                columns = (('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf), (gtk.CellRendererText(), TYPE_STRING)], True),
-                           (None, [(None, TYPE_INT)],                                                                  False),
-                           (None, [(None, TYPE_STRING)],                                                               False))
+                self.createTree()
 
-                self.tree = extTreeview.ExtTreeView(columns, True)
-
-                self.scrolled.add(self.tree)
-                self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_URI]])
-                self.tree.connect('drag-data-get', self.onDragDataGet)
-                self.tree.connect('key-press-event', self.onKeyPressed)
-                self.tree.connect('exttreeview-button-pressed', self.onMouseButton)
-                self.tree.connect('exttreeview-row-collapsed', self.onRowCollapsed)
-                self.tree.connect('exttreeview-row-expanded', self.onRowExpanded)
-
-            savedStates = prefs.get(__name__, 'saved-states', {})
-
-            # Save the current state if needed
+            # Save the state of the current root
             if self.currRoot is not None:
-                savedStates[self.currRoot] = {
-                                                'tree-state':     self.dumpTree(),
-                                                'selected-paths': self.tree.getSelectedPaths(),
-                                                'vscrollbar-pos': self.scrolled.get_vscrollbar().get_value(),
-                                                'hscrollbar-pos': self.scrolled.get_hscrollbar().get_value(),
-                                             }
-                prefs.set(__name__, 'saved-states', savedStates)
+                self.saveTreeState()
                 self.tree.clear()
 
-            self.currRoot = newRoot
+            self.currRoot = expName
 
-            if newRoot not in savedStates:
+            # Restore the state of the new root
+            if expName in self.treeState:
+                self.tree.handler_block_by_func(self.onRowExpanded)
+                self.restoreTreeDump(self.treeState[expName]['tree-state'])
+                self.tree.handler_unblock_by_func(self.onRowExpanded)
+
+                idle_add(self.scrolled.get_vscrollbar().set_value, self.treeState[expName]['vscrollbar-pos'])
+                idle_add(self.scrolled.get_hscrollbar().set_value, self.treeState[expName]['hscrollbar-pos'])
+                idle_add(self.tree.selectPaths, self.treeState[expName]['selected-paths'])
+                idle_add(self.refresh)
+            else:
                 self.exploreDir(None, self.folders[self.currRoot])
                 if len(self.tree) != 0:
                     self.tree.scroll_to_cell(0)
-            else:
-                savedState = savedStates[newRoot]
-
-                self.tree.handler_block_by_func(self.onRowExpanded)
-                self.restoreTree(savedState['tree-state'])
-                self.tree.handler_unblock_by_func(self.onRowExpanded)
-
-                idle_add(self.scrolled.get_vscrollbar().set_value, savedState['vscrollbar-pos'])
-                idle_add(self.scrolled.get_hscrollbar().set_value, savedState['hscrollbar-pos'])
-                idle_add(self.tree.selectPaths, savedState['selected-paths'])
-                idle_add(self.refresh)
-
-        elif msg == consts.MSG_EVT_APP_STARTED:
-            self.onModLoaded()
-        elif msg == consts.MSG_EVT_APP_QUIT:
-            self.onModUnloaded()
 
 
     # --== Configuration ==--
@@ -471,7 +464,9 @@ class FileExplorer(modules.Module):
     def configure(self, parent):
         """ Show the configuration dialog """
         if self.cfgWin is None:
-            self.cfgWin = gui.window.Window('FileExplorer.glade', 'vbox1', __name__, MOD_L10N, 370, 400)
+            from gui import extListview, window
+
+            self.cfgWin = window.Window('FileExplorer.glade', 'vbox1', __name__, MOD_L10N, 370, 400)
             # Create the list of folders
             txtRdr  = gtk.CellRendererText()
             pixRdr  = gtk.CellRendererPixbuf()
@@ -502,23 +497,27 @@ class FileExplorer(modules.Module):
 
     def populateFolderList(self):
         """ Populate the list of known folders """
-        self.cfgList.replaceContent([(name, consts.icoBtnDir, '<b>%s</b>\n<small>%s</small>' % (cgi.escape(name), cgi.escape(path)))
+        self.cfgList.replaceContent([(name, icons.dirBtnIcon(), '<b>%s</b>\n<small>%s</small>' % (tools.htmlEscape(name), tools.htmlEscape(path)))
                                      for name, path in sorted(self.folders.iteritems())])
 
 
     def onAddFolder(self, btn):
         """ Let the user add a new folder to the list """
+        from gui import selectPath
+
         result = selectPath.SelectPath(MOD_L10N, self.cfgWin, self.folders.keys()).run()
 
         if result is not None:
             name, path = result
             self.folders[name] = path
             self.populateFolderList()
-            modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': name, 'icon': None, 'widget': self.scrolled})
+            modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': name, 'icon': icons.dirMenuIcon(), 'widget': self.scrolled})
 
 
     def onRemoveSelectedFolder(self, list):
         """ Remove the selected media folder """
+        import gui
+
         if list.getSelectedRowsCount() == 1:
             remark   = _('You will be able to add this root folder again later on if you wish so.')
             question = _('Remove the selected entry?')
@@ -537,16 +536,16 @@ class FileExplorer(modules.Module):
                     self.currRoot = None
 
                 # Remove the saved state of the tree, if any
-                savedStates = prefs.get(__name__, 'saved-states', {})
-                if name in savedStates:
-                    del savedStates[name]
-                    prefs.set(__name__, 'saved-states', savedStates)
+                if name in self.treeState:
+                    del self.treeState[name]
 
             self.cfgList.removeSelectedRows()
 
 
     def onRenameFolder(self, btn):
         """ Let the user rename a folder """
+        from gui import selectPath
+
         name         = self.cfgList.getSelectedRows()[0][0]
         forbidden    = [rootName for rootName in self.folders if rootName != name]
         pathSelector = selectPath.SelectPath(MOD_L10N, self.cfgWin, forbidden)
@@ -580,7 +579,9 @@ class FileExplorer(modules.Module):
 
     def onHelp(self, btn):
         """ Display a small help message box """
-        helpDlg = help.HelpDlg(MOD_L10N)
+        import gui
+
+        helpDlg = gui.help.HelpDlg(MOD_L10N)
         helpDlg.addSection(_('Description'),
                            _('This module allows you to browse the files on your drives.'))
         helpDlg.addSection(_('Usage'),
