@@ -16,13 +16,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import dbus, gui, media.track, modules, traceback
+import modules, traceback
 
 from tools     import consts, prefs
 from gettext   import gettext as _
 from tools.log import logger
 
-MOD_INFO = ('Instant Messenger Status', _('Instant Messenger Status'), _('Update the status message of your IM client'), [], False, True)
+MOD_INFO = ('Instant Messenger Status', _('Instant Messenger Status'), _('Update the status message of your IM client'), [], False, True, consts.MODCAT_DESKTOP)
 MOD_NAME = MOD_INFO[modules.MODINFO_NAME]
 
 
@@ -185,34 +185,18 @@ class IMStatus(modules.Module):
 
     def __init__(self):
         """ Constructor """
-        modules.Module.__init__(self, (consts.MSG_EVT_NEW_TRACK,  consts.MSG_EVT_PAUSED,      consts.MSG_EVT_UNPAUSED,
-                                       consts.MSG_EVT_STOPPED,    consts.MSG_EVT_APP_STARTED, consts.MSG_EVT_APP_QUIT,
-                                       consts.MSG_EVT_MOD_LOADED, consts.MSG_EVT_MOD_UNLOADED))
+        handlers = {
+                        consts.MSG_EVT_PAUSED:       self.onPaused,
+                        consts.MSG_EVT_STOPPED:      self.onStopped,
+                        consts.MSG_EVT_UNPAUSED:     self.onUnpaused,
+                        consts.MSG_EVT_APP_QUIT:     self.onStopped,
+                        consts.MSG_EVT_NEW_TRACK:    self.onNewTrack,
+                        consts.MSG_EVT_MOD_LOADED:   self.onModLoaded,
+                        consts.MSG_EVT_APP_STARTED:  self.onModLoaded,
+                        consts.MSG_EVT_MOD_UNLOADED: self.onStopped,
+                   }
 
-
-    def init(self):
-        """ Initialize the module """
-        self.track     = None   # Current track
-        self.status    = ''     # The currently used status
-        self.paused    = False  # True if the current track is paused
-        self.clients   = []     # Clients currently active
-        self.cfgWindow = None   # Configuration window
-
-        # Detect active clients
-        try:
-            session        = dbus.SessionBus()
-            activeServices = session.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus').ListNames()
-            for activeClient in [client for client in CLIENTS if client[IM_DBUS_SERVICE_NAME] in activeServices]:
-                obj       = session.get_object(activeClient[IM_DBUS_SERVICE_NAME], activeClient[IM_DBUS_OBJECT_NAME])
-                interface = dbus.Interface(obj, activeClient[IM_DBUS_INTERFACE_NAME])
-
-                activeClient[IM_INSTANCE] = activeClient[IM_CLASS](interface)
-                activeClient[IM_ACCOUNTS] = activeClient[IM_INSTANCE].listAccounts()
-
-                logger.info('[%s] Found %s instance' % (MOD_NAME, activeClient[IM_NAME]))
-                self.clients.append(activeClient)
-        except:
-            logger.error('[%s] Error while initializing\n\n%s' % (MOD_NAME, traceback.format_exc()))
+        modules.Module.__init__(self, handlers)
 
 
     def __format(self, string, track):
@@ -232,12 +216,42 @@ class IMStatus(modules.Module):
 
 
     def setStatusMsg(self, status):
-        """ Try to update the status of all accounts of all active IM clients """
+        """ Update the status of all accounts of all active IM clients """
         for client in self.clients:
             for account in client[IM_ACCOUNTS]:
                 client[IM_INSTANCE].setStatusMsg(account, status)
 
 
+   # --== Message handlers ==--
+
+
+    def onModLoaded(self):
+        """ Initialize the module """
+        self.track     = None   # Current track
+        self.status    = ''     # The currently used status
+        self.paused    = False  # True if the current track is paused
+        self.clients   = []     # Clients currently active
+        self.cfgWindow = None   # Configuration window
+
+        # Detect active clients
+        try:
+            import dbus
+
+            session        = dbus.SessionBus()
+            activeServices = session.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus').ListNames()
+            for activeClient in [client for client in CLIENTS if client[IM_DBUS_SERVICE_NAME] in activeServices]:
+                obj       = session.get_object(activeClient[IM_DBUS_SERVICE_NAME], activeClient[IM_DBUS_OBJECT_NAME])
+                interface = dbus.Interface(obj, activeClient[IM_DBUS_INTERFACE_NAME])
+
+                activeClient[IM_INSTANCE] = activeClient[IM_CLASS](interface)
+                activeClient[IM_ACCOUNTS] = activeClient[IM_INSTANCE].listAccounts()
+
+                logger.info('[%s] Found %s instance' % (MOD_NAME, activeClient[IM_NAME]))
+                self.clients.append(activeClient)
+        except:
+            logger.error('[%s] Error while initializing\n\n%s' % (MOD_NAME, traceback.format_exc()))
+
+
     def onNewTrack(self, track):
         """ A new track is being played """
         self.track  = track
@@ -267,28 +281,15 @@ class IMStatus(modules.Module):
         self.setStatusMsg(self.status)
 
 
-    # --== Message handler ==--
-
-
-    def handleMsg(self, msg, params):
-        """ Handle messages sent to this module """
-        if   msg == consts.MSG_EVT_PAUSED:       self.onPaused()
-        elif msg == consts.MSG_EVT_STOPPED:      self.onStopped()
-        elif msg == consts.MSG_EVT_APP_QUIT:     self.onStopped()
-        elif msg == consts.MSG_EVT_UNPAUSED:     self.onUnpaused()
-        elif msg == consts.MSG_EVT_NEW_TRACK:    self.onNewTrack(params['track'])
-        elif msg == consts.MSG_EVT_MOD_LOADED:   self.init()
-        elif msg == consts.MSG_EVT_APP_STARTED:  self.init()
-        elif msg == consts.MSG_EVT_MOD_UNLOADED: self.onStopped()
-
-
     # --== Configuration ==--
 
 
     def configure(self, parent):
         """ Show the configuration window """
         if self.cfgWindow is None:
-            self.cfgWindow = gui.window.Window('IMStatus.glade', 'vbox1', __name__, _(MOD_NAME), 440, 290)
+            from gui.window import Window
+
+            self.cfgWindow = Window('IMStatus.glade', 'vbox1', __name__, _(MOD_NAME), 440, 290)
             # GTK handlers
             self.cfgWindow.getWidget('rad-stopDoNothing').connect('toggled', self.onRadToggled)
             self.cfgWindow.getWidget('rad-stopSetStatus').connect('toggled', self.onRadToggled)
@@ -337,6 +338,8 @@ class IMStatus(modules.Module):
 
     def onBtnHelp(self, btn):
         """ Display a small help message box """
+        import gui.help, media.track
+
         helpDlg = gui.help.HelpDlg(_(MOD_NAME))
         helpDlg.addSection(_('Description'),
                            _('This module detects any running instant messenger and updates your status with regards to the track '
