@@ -16,28 +16,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import cgi, collections, gtk, gui, media, modules, os, random, shutil, sys, tools
+import gtk, media, modules, os, tools
 
-from gui                   import fileChooser, help, questionMsgBox, extTreeview, extListview, progressDlg, selectPath
-from tools                 import consts, prefs, pickleLoad, pickleSave
+from tools                 import consts, htmlEscape, icons, prefs, pickleLoad, pickleSave
 from gettext               import ngettext, gettext as _
 from os.path               import isdir, isfile
 from gobject               import idle_add, TYPE_STRING, TYPE_INT, TYPE_PYOBJECT
 from tools.log             import logger
-from gui.progressDlg       import ProgressDlg
 from media.track.fileTrack import FileTrack
 
-MOD_INFO = ('Library', _('Library'), _('Organize your music by tags instead of files'), [], False, True)
+MOD_INFO = ('Library', _('Library'), _('Organize your music by tags'), [], False, True, consts.MODCAT_EXPLORER)
 MOD_L10N = MOD_INFO[modules.MODINFO_L10N]
-MOD_NAME = MOD_INFO[modules.MODINFO_NAME]
-
-
-# Constants
-VERSION                  = 3                                      # Used to enforce compatibility
-ROOT_PATH                = os.path.join(consts.dirCfg, 'Library') # Path where libraries are stored
-PREFS_DEFAULT_PREFIXES   = {'the ': None}                         # Prefixes are put at the end of artists' names
-PREFS_DEFAULT_LIBRARIES  = {}                                     # No libraries at first
-PREFS_DEFAULT_TREE_STATE = {}                                     # No state at first
 
 
 # Information associated with libraries
@@ -68,12 +57,13 @@ PREFS_DEFAULT_TREE_STATE = {}                                     # No state at
 
 # Possible types for a node of the tree
 (
-    TYPE_ARTIST,    # Artist
-    TYPE_ALBUM,     # Album
-    TYPE_TRACK,     # Single track
-    TYPE_HEADER,    # Alphabetical header
-    TYPE_NONE       # Used for fake children
-) = range(5)
+    TYPE_ARTIST,            # Artist
+    TYPE_ALBUM,             # Album
+    TYPE_TRACK,             # Single track
+    TYPE_HEADER,            # Alphabetical header
+    TYPE_FAVORITES_BANNER,  # Favorites banner (when showing only favorites)
+    TYPE_NONE               # Used for fake children
+) = range(6)
 
 
 # The format of a row in the treeview
@@ -83,43 +73,40 @@ PREFS_DEFAULT_TREE_STATE = {}                                     # No state at
     ROW_NAME,      # Item name
     ROW_TYPE,      # The type of the item (e.g., directory, file)
     ROW_FULLPATH,  # The full path to the item
-    ROW_TAGS       # The tags of the track, valid only for rows of type TYPE_TRACK
+    ROW_DATA       # Arbitrary data that depend on the type of the row
 ) = range(6)
 
 
+# Constants
+VERSION                           = 3                                      # Used to check compatibility
+ROOT_PATH                         = os.path.join(consts.dirCfg, 'Library') # Path where libraries are stored
+FAKE_CHILD                        = (None, None, '', TYPE_NONE, '', None)  # We use a lazy tree
+PREFS_DEFAULT_PREFIXES            = {'the ': None}                         # Prefixes are put at the end of artists' names
+PREFS_DEFAULT_LIBRARIES           = {}                                     # No libraries at first
+PREFS_DEFAULT_TREE_STATE          = {}                                     # No state at first
+PREFS_DEFAULT_SHOW_ONLY_FAVORITES = False
+
+
 class Library(modules.Module):
 
 
     def __init__(self):
         """ Constructor """
-        modules.Module.__init__(self, (consts.MSG_EVT_APP_STARTED, consts.MSG_EVT_EXPLORER_CHANGED, consts.MSG_EVT_MOD_LOADED,
-                                       consts.MSG_EVT_APP_QUIT,    consts.MSG_EVT_MOD_UNLOADED))
+        handlers = {
+                        consts.MSG_EVT_APP_QUIT:         self.onModUnloaded,
+                        consts.MSG_EVT_MOD_LOADED:       self.onModLoaded,
+                        consts.MSG_EVT_APP_STARTED:      self.onModLoaded,
+                        consts.MSG_EVT_MOD_UNLOADED:     self.onModUnloaded,
+                        consts.MSG_EVT_EXPLORER_CHANGED: self.onExplorerChanged,
+                   }
 
-
-    def onAppStarted(self):
-        """ This is the real initialization function, called when the module has been loaded """
-        self.tree      = None
-        self.currLib   = None
-        self.cfgWindow = None
-        self.libraries = prefs.get(__name__, 'libraries',  PREFS_DEFAULT_LIBRARIES)
-        self.treeState = prefs.get(__name__, 'tree-state', PREFS_DEFAULT_TREE_STATE)
-        # Scroll window
-        self.scrolled = gtk.ScrolledWindow()
-        self.scrolled.set_shadow_type(gtk.SHADOW_IN)
-        self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.scrolled.show()
-
-
-    def isDraggable(self):
-        """ Only playable rows can be dragged """
-        for row in self.tree.iterSelectedRows():
-            if row[ROW_TYPE] != TYPE_NONE:
-                return True
-        return False
+        modules.Module.__init__(self, handlers)
 
 
     def __createTree(self):
         """ Create the main tree, add it to the scrolled window """
+        from gui.extTreeview import ExtTreeView
+
         txtRdr         = gtk.CellRendererText()
         pixbufRdr      = gtk.CellRendererPixbuf()
         txtRdrAlbumLen = gtk.CellRendererText()
@@ -129,7 +116,7 @@ class Library(modules.Module):
                    (None, [(None, TYPE_STRING)],                                                               False),
                    (None, [(None, TYPE_PYOBJECT)],                                                             False))
 
-        self.tree = extTreeview.ExtTreeView(columns, True)
+        self.tree = ExtTreeView(columns, True)
 
         # The first text column (ROW_ALBUM_LEN) is not the one to search for
         # set_search_column(ROW_NAME) should work, but it doesn't...
@@ -141,14 +128,14 @@ class Library(modules.Module):
 
         # The album length is written in a smaller font, with a lighter color
         txtRdrAlbumLen.set_property('scale', 0.85)
-        txtRdrAlbumLen.set_property('foreground', '#909090')
+        txtRdrAlbumLen.set_property('foreground-gdk', self.tree.get_style().text[gtk.STATE_INSENSITIVE])
 
-        self.tree.setIsDraggableFunc(self.isDraggable)
         self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_TRACKS]])
         # GTK handlers
         self.tree.connect('drag-data-get',              self.onDragDataGet)
         self.tree.connect('key-press-event',            self.onKeyPressed)
         self.tree.connect('exttreeview-row-expanded',   self.onRowExpanded)
+        self.tree.connect('exttreeview-row-collapsed',  self.onRowCollapsed)
         self.tree.connect('exttreeview-button-pressed', self.onButtonPressed)
         # Add the tree to the scrolled window
         self.scrolled.add(self.tree)
@@ -173,6 +160,8 @@ class Library(modules.Module):
 
     def __createEmptyLibrary(self, name):
         """ Create bootstrap files for a new library """
+        import shutil
+
         # Make sure that the root directory of all libraries exists
         if not isdir(ROOT_PATH):
             os.mkdir(ROOT_PATH)
@@ -186,11 +175,15 @@ class Library(modules.Module):
 
     def refreshLibrary(self, parent, libName, path, creation=False):
         """ Refresh the given library, must be called through idle_add() """
+        import collections, shutil
+
+        from gui import progressDlg
+
         # First show a progress dialog
         if creation: header = _('Creating library')
         else:        header = _('Refreshing library')
 
-        progress = ProgressDlg(parent, header, _('The directory is scanned for media files. This can take some time.\nPlease wait.'))
+        progress = progressDlg.ProgressDlg(parent, header, _('The directory is scanned for media files. This can take some time.\nPlease wait.'))
         yield True
 
         libPath = os.path.join(ROOT_PATH, libName)   # Location of the library
@@ -285,6 +278,10 @@ class Library(modules.Module):
         progress.pulse()
         yield True
 
+        # Load favorites before removing the files from the disk
+        if self.currLib == libName: favorites = self.favorites
+        else:                       favorites = self.loadFavorites(libName)
+
         # Re-create the library structure on the disk
         if isdir(libPath):
             shutil.rmtree(libPath)
@@ -323,14 +320,26 @@ class Library(modules.Module):
         self.libraries[libName] = (path, overallNbArtists, overallNbAlbums, overallNbTracks)
         self.fillLibraryList()
         if creation:
-            modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': libName, 'icon': None, 'widget': self.scrolled})
+            modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': libName, 'icon': icons.dirMenuIcon(), 'widget': self.scrolled})
         progress.destroy()
 
+        # Trim favorites and save them
+        newFavorites = {}
+        for (artist, albums) in favorites.iteritems():
+            if artist in db:
+                newFavorites[artist] = {}
+                for album in albums:
+                    if album in db[artist]:
+                        newFavorites[artist][album] = None
+
+        self.saveFavorites(libName, newFavorites)
+
         # If the refreshed library is currently displayed, refresh the treeview as well
         if self.currLib == libName:
-            treeState = self.tree.saveState(ROW_NAME)
-            self.loadLibrary(self.tree, self.currLib)
-            self.tree.restoreState(treeState, ROW_NAME)
+            self.saveTreeState()
+            self.favorites = newFavorites
+            self.loadArtists(self.tree, self.currLib)
+            self.restoreTreeState()
 
         yield False
 
@@ -341,6 +350,8 @@ class Library(modules.Module):
                 * The list 'paths' if it is not None
                 * The currently selected rows if 'paths' is None
         """
+        from sys import maxint
+
         tracks = []
 
         if paths is None:
@@ -349,14 +360,14 @@ class Library(modules.Module):
         for currPath in paths:
             row = tree.getRow(currPath)
             if row[ROW_TYPE] == TYPE_TRACK:
-                tracks.append(row[ROW_TAGS])
+                tracks.append(row[ROW_DATA])
             elif row[ROW_TYPE] == TYPE_ALBUM:
                 tracks.extend(pickleLoad(row[ROW_FULLPATH]))
             elif row[ROW_TYPE] == TYPE_ARTIST:
                 for album in pickleLoad(os.path.join(row[ROW_FULLPATH], 'albums')):
                     tracks.extend(pickleLoad(os.path.join(row[ROW_FULLPATH], album[ALB_INDEX])))
             elif row[ROW_TYPE] == TYPE_HEADER:
-                for path in xrange(currPath[0]+1, sys.maxint):
+                for path in xrange(currPath[0]+1, maxint):
                     if not tree.isValidPath(path):
                         break
 
@@ -378,11 +389,13 @@ class Library(modules.Module):
         tracks = self.__getTracksFromPaths(tree, paths)
 
         if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
-        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
+        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks, 'playNow': False})
 
 
     def pickAlbumArtist(self, tree, artistPath):
         """ Pick an album at random of the given artist and play it """
+        import random
+
         # Expanding the artist row populates it, so that we can then pick an album at random
         tree.expandRow(artistPath)
         albumPath = artistPath + (random.randint(0, tree.getNbChildren(artistPath)-1), )
@@ -396,40 +409,111 @@ class Library(modules.Module):
 
     def pickAlbumLibrary(self, tree):
         """ Pick an album at random in the library and play it """
+        import random
+
         # Pick an artist at random (make sure not to select an alphabetical header)
-        while True:
+        path = (random.randint(0, tree.getCount()-1), )
+        while tree.getItem(path, ROW_TYPE) != TYPE_ARTIST:
             path = (random.randint(0, tree.getCount()-1), )
-            if tree.getItem(path, ROW_TYPE) == TYPE_ARTIST:
-                break
 
         self.pickAlbumArtist(tree, path)
 
 
+    def switchFavoriteStateOfSelectedItems(self, tree):
+        """ Add to/remove from favorites the selected items """
+        # Go through selected items and switch their state
+        removed = False
+        for path in tree.getSelectedPaths():
+            if tree.getItem(path, ROW_TYPE) == TYPE_ALBUM:
+                album  = tree.getItem(path, ROW_DATA)
+                artist = tree.getItem(path[:-1], ROW_DATA)
+
+                if self.isAlbumInFavorites(artist, album):
+                    removed = True
+                    tree.setItem(path, ROW_PIXBUF, icons.mediaDirMenuIcon())
+                    self.removeFromFavorites(artist, album)
+                else:
+                    tree.setItem(path, ROW_PIXBUF, icons.starDirMenuIcon())
+                    self.addToFavorites(artist, album)
+
+        # If some favorites were removed, we may have to reload the tree
+        if self.showOnlyFavorites and removed:
+            self.saveTreeState()
+            self.loadArtists(self.tree, self.currLib)
+            self.restoreTreeState()
+
+
+    def switchFavoritesView(self, tree):
+        """ Show all/favorites """
+        self.saveTreeState()
+        self.showOnlyFavorites = not self.showOnlyFavorites
+        prefs.set(__name__, 'show-only-favorites', self.showOnlyFavorites)
+        self.loadArtists(self.tree, self.currLib)
+        self.restoreTreeState()
+
+
     def showPopupMenu(self, tree, button, time, path):
         """ Show a popup menu """
-        popup    = gtk.Menu()
-        playable = path is not None and tree.getItem(path, ROW_TYPE) != TYPE_NONE
+        popup = gtk.Menu()
 
-        # Play selection
+        # Play
         play = gtk.ImageMenuItem(gtk.STOCK_MEDIA_PLAY)
-        play.set_sensitive(playable)
-        play.connect('activate', lambda widget: self.playPaths(tree, None, True))
         popup.append(play)
 
-        # Add selection
-        add = gtk.ImageMenuItem(gtk.STOCK_ADD)
-        add.set_sensitive(playable)
-        add.connect('activate', lambda widget: self.playPaths(tree, None, False))
-        popup.append(add)
+        if path is None: play.set_sensitive(False)
+        else:            play.connect('activate', lambda widget: self.playPaths(tree, None, True))
 
+        # Separator
+        popup.append(gtk.SeparatorMenuItem())
+
+        # Add to/remove from favorites
+        favCpt    = 0
+        nonFavCpt = 0
+        for node in tree.getSelectedPaths():
+            if tree.getItem(node, ROW_TYPE) != TYPE_ALBUM:
+                favCpt    = 1
+                nonFavCpt = 1
+                break
+            elif tree.getItem(node, ROW_PIXBUF) == icons.mediaDirMenuIcon():
+                nonFavCpt += 1
+            else:
+                favCpt += 1
+
+        if favCpt == 0:      favorite = gtk.ImageMenuItem(_('Add to Favorites'))
+        elif nonFavCpt == 0: favorite = gtk.ImageMenuItem(_('Remove from Favorites'))
+        else:                favorite = gtk.ImageMenuItem(_('Favorites'))
+
+        favorite.get_image().set_from_pixbuf(icons.starMenuIcon())
+        popup.append(favorite)
+
+        if TYPE_ALBUM in [tree.getItem(path, ROW_TYPE) for path in tree.getSelectedPaths()]:
+            favorite.connect('activate', lambda widget: self.switchFavoriteStateOfSelectedItems(tree))
+        else:
+            favorite.set_sensitive(False)
+
+        # Show only favorites
+        showFavorites = gtk.CheckMenuItem(_('Show Only Favorites'))
+        showFavorites.set_active(self.showOnlyFavorites)
+        showFavorites.connect('toggled', lambda widget: self.switchFavoritesView(tree))
+        popup.append(showFavorites)
+
+        # Separator
         popup.append(gtk.SeparatorMenuItem())
 
         # Collapse all nodes
         collapse = gtk.ImageMenuItem(_('Collapse all'))
         collapse.set_image(gtk.image_new_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_MENU))
-        collapse.connect('activate', lambda widget: self.tree.collapse_all())
         popup.append(collapse)
 
+        enabled = False
+        for child in self.tree.iterChildren(None):
+            if self.tree.row_expanded(child):
+                enabled = True
+                break
+
+        if enabled: collapse.connect('activate', lambda widget: self.tree.collapse_all())
+        else:       collapse.set_sensitive(False)
+
         # Refresh the library
         refresh = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
         refresh.connect('activate', lambda widget: idle_add(self.refreshLibrary(None, self.currLib, self.libraries[self.currLib][LIB_PATH]).next))
@@ -457,22 +541,32 @@ class Library(modules.Module):
         popup.popup(None, None, None, button, time)
 
 
-    def loadLibrary(self, tree, name):
+    # --== Populating the tree ==--
+
+
+    def loadArtists(self, tree, name):
         """ Load the given library """
-        rows     = []
-        path     = os.path.join(ROOT_PATH, name)
-        prevChar = ''
+        libPath = os.path.join(ROOT_PATH, name)
 
         # Make sure the version number is the good one
-        if not os.path.exists(os.path.join(path, 'VERSION_%u' % VERSION)):
-            logger.error('[%s] Version number does not match, loading of library "%s" aborted' % (MOD_NAME, name))
+        if not os.path.exists(os.path.join(libPath, 'VERSION_%u' % VERSION)):
+            logger.error('[%s] Version number does not match, loading of library "%s" aborted' % (MOD_INFO[modules.MODINFO_NAME], name))
             error = _('This library is deprecated, please refresh it.')
-            tree.replaceContent([(consts.icoError, None, error, TYPE_NONE, None, None)])
+            tree.replaceContent([(icons.errorMenuIcon(), None, error, TYPE_NONE, None, None)])
             return
 
-        # Create the rows, with alphabetical header if needed
-        for artist in pickleLoad(os.path.join(path, 'artists')):
+        rows       = []
+        icon       = icons.dirMenuIcon()
+        prevChar   = ''
+        allArtists = pickleLoad(os.path.join(libPath, 'artists'))
 
+        # Filter artists if needed
+        if self.showOnlyFavorites:
+            allArtists = [artist for artist in allArtists if self.isArtistInFavorites(artist[ART_NAME])]
+            rows.append((icons.starMenuIcon(), None, '<b>%s</b>' % _('My Favorites'), TYPE_FAVORITES_BANNER, None, None))
+
+        # Create the rows
+        for artist in allArtists:
             if len(artist[ART_NAME]) != 0: currChar = unicode(artist[ART_NAME], errors='replace')[0].lower()
             else:                          currChar = prevChar
 
@@ -481,48 +575,137 @@ class Library(modules.Module):
                 if currChar.isdigit(): rows.append((None, None, '<b>0 - 9</b>',                 TYPE_HEADER, None, None))
                 else:                  rows.append((None, None, '<b>%s</b>' % currChar.upper(), TYPE_HEADER, None, None))
 
-            rows.append((consts.icoDir, None, cgi.escape(artist[ART_NAME]), TYPE_ARTIST, os.path.join(path, artist[ART_INDEX]), None))
+            rows.append((icon, None, htmlEscape(artist[ART_NAME]), TYPE_ARTIST, os.path.join(libPath, artist[ART_INDEX]), artist[ART_NAME]))
 
         # Insert all rows, and then add a fake child to each artist
         tree.replaceContent(rows)
         for node in tree.iterChildren(None):
             if tree.getItem(node, ROW_TYPE) == TYPE_ARTIST:
-                tree.appendRow((None, None, '', TYPE_NONE, '', None), node)
+                tree.appendRow(FAKE_CHILD, node)
 
 
     def loadAlbums(self, tree, node, fakeChild):
-        """ Initial load of all albums of the given node, assuming it is of type TYPE_ARTIST """
-        allAlbums = pickleLoad(os.path.join(tree.getItem(node, ROW_FULLPATH), 'albums'))
+        """ Initial load of the albums of the given node, assuming it is of type TYPE_ARTIST """
+        rows      = []
         path      = tree.getItem(node, ROW_FULLPATH)
-        rows      = [(consts.icoMediaDir, '[%s]' % tools.sec2str(album[ALB_LENGTH], True), '%s' % cgi.escape(album[ALB_NAME]), TYPE_ALBUM, os.path.join(path, album[ALB_INDEX]), None) for album in allAlbums]
+        artist    = tree.getItem(node, ROW_DATA)
+        allAlbums = pickleLoad(os.path.join(tree.getItem(node, ROW_FULLPATH), 'albums'))
+
+        # Filter albums if only favorites should be shown
+        if self.showOnlyFavorites:
+            allAlbums = [album for album in allAlbums if self.isAlbumInFavorites(artist, album[ALB_NAME])]
+
+        # The icon depends on whether the album is in the favorites
+        for album in allAlbums:
+            if self.isAlbumInFavorites(artist, album[ALB_NAME]): icon = icons.starDirMenuIcon()
+            else:                                                icon = icons.mediaDirMenuIcon()
+
+            rows.append((icon, '[%s]' % tools.sec2str(album[ALB_LENGTH], True), '%s' % htmlEscape(album[ALB_NAME]),
+                            TYPE_ALBUM, os.path.join(path, album[ALB_INDEX]), album[ALB_NAME]))
 
         # Add all the rows, and then add a fake child to each of them
-        tree.freeze_child_notify()
         tree.appendRows(rows, node)
         tree.removeRow(fakeChild)
         for child in tree.iterChildren(node):
-            tree.appendRow((None, None, '', TYPE_NONE, '', None), child)
-        tree.thaw_child_notify()
+            tree.appendRow(FAKE_CHILD, child)
 
 
     def loadTracks(self, tree, node, fakeChild):
         """ Initial load of all tracks of the given node, assuming it is of type TYPE_ALBUM """
         allTracks = pickleLoad(tree.getItem(node, ROW_FULLPATH))
-        rows      = [(consts.icoMediaFile, None, '%02u. %s' % (track.getNumber(), cgi.escape(track.getTitle())), TYPE_TRACK, track.getFilePath(), track) for track in allTracks]
+        icon      = icons.mediaFileMenuIcon()
+        rows      = [(icon, None, '%02u. %s' % (track.getNumber(), htmlEscape(track.getTitle())), TYPE_TRACK, track.getFilePath(), track) for track in allTracks]
 
         tree.appendRows(rows, node)
         tree.removeRow(fakeChild)
 
 
+    # --== Manage tree state ==--
+
+
+    def saveTreeState(self):
+        """ Save the current tree state """
+        if self.showOnlyFavorites: self.treeStates[self.currLib + ' favorites'] = self.tree.saveState(ROW_NAME)
+        else:                      self.treeStates[self.currLib]                = self.tree.saveState(ROW_NAME)
+
+
+    def restoreTreeState(self):
+        """ Restore the tree state """
+        if self.showOnlyFavorites: name = self.currLib + ' favorites'
+        else:                      name = self.currLib
+
+        if name in self.treeStates:
+            self.tree.restoreState(self.treeStates[name], ROW_NAME)
+
+
+    def removeTreeStates(self, libName):
+        """ Remove the tree states associated to the given library """
+        if libName in self.treeStates:                del self.treeStates[libName]
+        if libName + ' favorites' in self.treeStates: del self.treeStates[libName + ' favorites']
+
+
+    def renameTreeStates(self, oldLibName, newLibName):
+        """ Rename the tree states associated with oldLibName """
+        if oldLibName in self.treeStates:
+            self.treeStates[newLibName] = self.treeStates[oldLibName]
+            del self.treeStates[oldLibName]
+
+        if oldLibName + ' favorites' in self.treeStates:
+            self.treeStates[newLibName + ' favorites'] = self.treeStates[oldLibName + ' favorites']
+            del self.treeStates[oldLibName + ' favorites']
+
+
+    # --== Favorites ==--
+
+
+    def loadFavorites(self, libName):
+        """ Load favorites from the disk """
+        try:    return pickleLoad(os.path.join(ROOT_PATH, libName, 'favorites'))
+        except: return {}
+
+
+    def saveFavorites(self, libName, favorites):
+        """ Save favorites to the disk """
+        pickleSave(os.path.join(ROOT_PATH, libName, 'favorites'), favorites)
+
+
+    def isArtistInFavorites(self, artist):
+        """ Return whether the given artist is in the favorites (at least one album) """
+        return artist in self.favorites
+
+
+    def isAlbumInFavorites(self, artist, album):
+        """ Return whether the given album is in the favorites """
+        return artist in self.favorites and album in self.favorites[artist]
+
+
+    def addToFavorites(self, artist, album):
+        """ Add the given album to the favorites """
+        if artist in self.favorites: self.favorites[artist][album] = None
+        else:                        self.favorites[artist] = {album: None}
+
+
+    def removeFromFavorites(self, artist, album):
+        """ Remove the given album from the favorites """
+        del self.favorites[artist][album]
+
+        if len(self.favorites[artist]) == 0:
+            del self.favorites[artist]
+
+
     # --== GTK handlers ==--
 
 
     def onRowExpanded(self, tree, node):
-        """ Populate the expanded row if needed (e.g., it still has only a fake child) """
-        child = tree.getChild(node, 0)
-        if tree.getItem(child, ROW_TYPE) == TYPE_NONE:
-            if tree.getItem(node, ROW_TYPE) == TYPE_ARTIST: self.loadAlbums(tree, node, child)
-            else:                                           self.loadTracks(tree, node, child)
+        """ Populate the expanded row """
+        if tree.getItem(node, ROW_TYPE) == TYPE_ARTIST: self.loadAlbums(tree, node, tree.getChild(node, 0))
+        else:                                           self.loadTracks(tree, node, tree.getChild(node, 0))
+
+
+    def onRowCollapsed(self, tree, node):
+        """ Replace all children of the node by a fake child """
+        tree.removeAllChildren(node)
+        tree.appendRow(FAKE_CHILD, node)
 
 
     def onButtonPressed(self, tree, event, path):
@@ -533,16 +716,16 @@ class Library(modules.Module):
             if event.button == 2:
                 self.playPaths(tree, [path], False)
             elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
-                if   tree.getItem(path, ROW_PIXBUF) != consts.icoDir: self.playPaths(tree, None, True)
-                elif tree.row_expanded(path):                         tree.collapse_row(path)
-                else:                                                 tree.expand_row(path, False)
+                if   tree.getItem(path, ROW_PIXBUF) != icons.dirMenuIcon(): self.playPaths(tree, None, True)
+                elif tree.row_expanded(path):                               tree.collapse_row(path)
+                else:                                                       tree.expand_row(path, False)
 
 
     def onKeyPressed(self, tree, event):
         """ A key has been pressed """
         keyname = gtk.gdk.keyval_name(event.keyval)
 
-        if keyname == 'F5':       idle_add(self.refreshLibrary(None, self.currLib, self.libraries[self.currLib][LIB_PATH]).next)
+        if   keyname == 'F5':     idle_add(self.refreshLibrary(None, self.currLib, self.libraries[self.currLib][LIB_PATH]).next)
         elif keyname == 'plus':   tree.expandRows()
         elif keyname == 'Left':   tree.collapseRows()
         elif keyname == 'Right':  tree.expandRows()
@@ -560,7 +743,7 @@ class Library(modules.Module):
     def addAllExplorers(self):
         """ Add all libraries to the Explorer module """
         for (name, (path, nbArtists, nbAlbums, nbTracks)) in self.libraries.iteritems():
-            modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': name, 'icon': None, 'widget': self.scrolled})
+            modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': name, 'icon': icons.dirMenuIcon(), 'widget': self.scrolled})
 
 
     def removeAllExplorers(self):
@@ -569,39 +752,55 @@ class Library(modules.Module):
             modules.postMsg(consts.MSG_CMD_EXPLORER_REMOVE, {'modName': MOD_L10N, 'expName': name})
 
 
-   # --== Message handler ==--
+   # --== Message handlers ==--
 
 
-    def handleMsg(self, msg, params):
-        """ Handle messages sent to this module """
-        if msg == consts.MSG_EVT_APP_STARTED or msg == consts.MSG_EVT_MOD_LOADED:
-            self.onAppStarted()
-            idle_add(self.addAllExplorers)
+    def onModLoaded(self):
+        """ This is the real initialization function, called when the module has been loaded """
+        self.tree              = None
+        self.currLib           = None
+        self.cfgWindow         = None
+        self.libraries         = prefs.get(__name__, 'libraries',  PREFS_DEFAULT_LIBRARIES)
+        self.favorites         = None
+        self.treeStates        = prefs.get(__name__, 'tree-state', PREFS_DEFAULT_TREE_STATE)
+        self.showOnlyFavorites = prefs.get(__name__, 'show-only-favorites', PREFS_DEFAULT_SHOW_ONLY_FAVORITES)
+        # Scroll window
+        self.scrolled = gtk.ScrolledWindow()
+        self.scrolled.set_shadow_type(gtk.SHADOW_IN)
+        self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.scrolled.show()
 
-        elif msg == consts.MSG_EVT_EXPLORER_CHANGED and params['modName'] == MOD_L10N and params['expName'] != self.currLib:
+        idle_add(self.addAllExplorers)
+
+
+    def onModUnloaded(self):
+        """ The module has been unloaded """
+        if self.currLib is not None:
+            self.saveTreeState()
+            self.saveFavorites(self.currLib, self.favorites)
+            prefs.set(__name__, 'tree-state', self.treeStates)
+
+        prefs.set(__name__, 'libraries',  self.libraries)
+        self.removeAllExplorers()
+
+
+    def onExplorerChanged(self, modName, expName):
+        """ A new explorer has been selected """
+        if modName == MOD_L10N and expName != self.currLib:
             # Create the tree if needed
             if self.tree is None:
                 self.__createTree()
 
             # Save the state of the current library
             if self.currLib is not None:
-                self.treeState[self.currLib] = self.tree.saveState(ROW_NAME)
+                self.saveTreeState()
+                self.saveFavorites(self.currLib, self.favorites)
 
-            # Switch to the new one
-            self.currLib = params['expName']
-            self.loadLibrary(self.tree, self.currLib)
-
-            # Restore the state of the new library
-            if len(self.tree) != 0 and self.currLib in self.treeState:
-                self.tree.restoreState(self.treeState[self.currLib], ROW_NAME)
-
-        elif msg == consts.MSG_EVT_APP_QUIT or msg == consts.MSG_EVT_MOD_UNLOADED:
-            if self.currLib is not None:
-                self.treeState[self.currLib] = self.tree.saveState(ROW_NAME)
-                prefs.set(__name__, 'tree-state', self.treeState)
-
-            prefs.set(__name__, 'libraries',  self.libraries)
-            self.removeAllExplorers()
+            # Switch to the new library
+            self.currLib   = expName
+            self.favorites = self.loadFavorites(self.currLib)
+            self.loadArtists(self.tree, self.currLib)
+            self.restoreTreeState()
 
 
     # --== Configuration ==--
@@ -610,7 +809,9 @@ class Library(modules.Module):
     def configure(self, parent):
         """ Show the configuration dialog """
         if self.cfgWindow is None:
-            self.cfgWindow = gui.window.Window('Library.glade', 'vbox1', __name__, MOD_L10N, 370, 400)
+            from gui import extListview, window
+
+            self.cfgWindow = window.Window('Library.glade', 'vbox1', __name__, MOD_L10N, 370, 400)
             # Create the list of libraries
             txtRdr  = gtk.CellRendererText()
             pixRdr  = gtk.CellRendererPixbuf()
@@ -646,7 +847,9 @@ class Library(modules.Module):
 
     def onAddLibrary(self, btn):
         """ Let the user create a new library """
-        result = selectPath.SelectPath(MOD_L10N, self.cfgWindow, self.libraries.keys(), ['/']).run()
+        from gui.selectPath import SelectPath
+
+        result = SelectPath(MOD_L10N, self.cfgWindow, self.libraries.keys(), ['/']).run()
 
         if result is not None:
             name, path = result
@@ -655,6 +858,8 @@ class Library(modules.Module):
 
     def renameLibrary(self, oldName, newName):
         """ Rename a library """
+        import shutil
+
         self.libraries[newName] = self.libraries[oldName]
         del self.libraries[oldName]
 
@@ -662,14 +867,23 @@ class Library(modules.Module):
         newPath = os.path.join(ROOT_PATH, newName)
         shutil.move(oldPath, newPath)
 
+        # Rename tree states as well
+        self.renameTreeStates(oldName, newName)
+
+        # Is it the current library?
+        if self.currLib == oldName:
+            self.currLib = newName
+
         modules.postMsg(consts.MSG_CMD_EXPLORER_RENAME, {'modName': MOD_L10N, 'expName': oldName, 'newExpName': newName})
 
 
     def onRenameLibrary(self, btn):
         """ Let the user rename a library """
+        from gui.selectPath import SelectPath
+
         name         = self.cfgList.getSelectedRows()[0][0]
         forbidden    = [libName for libName in self.libraries if libName != name]
-        pathSelector = selectPath.SelectPath(MOD_L10N, self.cfgWindow, forbidden, ['/'])
+        pathSelector = SelectPath(MOD_L10N, self.cfgWindow, forbidden, ['/'])
 
         pathSelector.setPathSelectionEnabled(False)
         result = pathSelector.run(name, self.libraries[name][LIB_PATH])
@@ -682,13 +896,17 @@ class Library(modules.Module):
     def fillLibraryList(self):
         """ Fill the list of libraries """
         if self.cfgWindow is not None:
-            rows = [(name, consts.icoBtnDir, '<b>%s</b>\n<small>%s - %u %s</small>' % (cgi.escape(name), cgi.escape(path), nbTracks, cgi.escape(_('tracks'))))
+            rows = [(name, icons.dirBtnIcon(), '<b>%s</b>\n<small>%s - %u %s</small>' % (htmlEscape(name), htmlEscape(path), nbTracks, htmlEscape(_('tracks'))))
                     for name, (path, nbArtists, nbAlbums, nbTracks) in sorted(self.libraries.iteritems())]
             self.cfgList.replaceContent(rows)
 
 
     def removeSelectedLibraries(self, list):
         """ Remove all selected libraries """
+        import shutil
+
+        from gui import questionMsgBox
+
         if list.getSelectedRowsCount() == 1:
             remark   = _('You will be able to recreate this library later on if you wish so.')
             question = _('Remove the selected library?')
@@ -698,13 +916,20 @@ class Library(modules.Module):
 
         if questionMsgBox(self.cfgWindow, question, '%s %s' % (_('Your media files will not be removed.'), remark)) == gtk.RESPONSE_YES:
             for row in list.getSelectedRows():
+                libName = row[0]
+
+                if self.currLib == libName:
+                    self.currLib = None
+
                 # Remove the library from the disk
-                libPath = os.path.join(ROOT_PATH, row[0])
+                libPath = os.path.join(ROOT_PATH, libName)
                 if isdir(libPath):
                     shutil.rmtree(libPath)
                 # Remove the corresponding explorer
-                modules.postMsg(consts.MSG_CMD_EXPLORER_REMOVE, {'modName': MOD_L10N, 'expName': row[0]})
-                del self.libraries[row[0]]
+                modules.postMsg(consts.MSG_CMD_EXPLORER_REMOVE, {'modName': MOD_L10N, 'expName': libName})
+                del self.libraries[libName]
+                # Remove tree states
+                self.removeTreeStates(libName)
             # Clean up the listview
             list.removeSelectedRows()
 
@@ -724,6 +949,8 @@ class Library(modules.Module):
 
     def onHelp(self, btn):
         """ Display a small help message box """
+        from gui import help
+
         helpDlg = help.HelpDlg(MOD_L10N)
         helpDlg.addSection(_('Description'),
                            _('This module organizes your media files by tags instead of using the file structure of your drive. '
