@@ -16,49 +16,32 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import gtk, modules, os.path
+import gtk, modules
 
 from tools   import consts, loadGladeFile, prefs
 from gettext import gettext as _
 
-MOD_INFO = ('Status Icon', _('Status Icon'), _('Add an icon to the notification area'), [], False, False)
+MOD_INFO = ('Status Icon', _('Status Icon'), _('Add an icon to the notification area'), [], False, False, consts.MODCAT_DESKTOP)
 
 
 class StatusIcon(modules.Module):
 
     def __init__(self):
         """ Constructor """
-        modules.Module.__init__(self, (consts.MSG_EVT_MOD_LOADED, consts.MSG_EVT_MOD_UNLOADED,  consts.MSG_EVT_APP_STARTED,
-                                       consts.MSG_EVT_NEW_TRACK,  consts.MSG_EVT_PAUSED,        consts.MSG_EVT_UNPAUSED,
-                                       consts.MSG_EVT_STOPPED,    consts.MSG_EVT_NEW_TRACKLIST, consts.MSG_EVT_TRACK_MOVED))
+        handlers = {
+                        consts.MSG_EVT_PAUSED:         self.onPaused,
+                        consts.MSG_EVT_STOPPED:        self.onStopped,
+                        consts.MSG_EVT_UNPAUSED:       self.onUnpaused,
+                        consts.MSG_EVT_NEW_TRACK:      self.onNewTrack,
+                        consts.MSG_EVT_MOD_LOADED:     self.onModLoaded,
+                        consts.MSG_EVT_APP_STARTED:    self.onModLoaded,
+                        consts.MSG_EVT_TRACK_MOVED:    self.onTrackMoved,
+                        consts.MSG_EVT_MOD_UNLOADED:   self.onModUnloaded,
+                        consts.MSG_EVT_NEW_TRACKLIST:  self.onNewTracklist,
+                        consts.MSG_EVT_VOLUME_CHANGED: self.onVolumeChanged,
+                   }
 
-
-    def install(self):
-        """ Install the Status icon """
-        self.tooltip          = consts.appName
-        self.isPaused         = False
-        self.popupMenu        = None
-        self.isPlaying        = False
-        self.icoNormal        = None
-        self.statusIcon       = gtk.StatusIcon()
-        self.mainWindow       = prefs.getWidgetsTree().get_widget('win-main')
-        self.trackHasNext     = False
-        self.trackHasPrev     = False
-        self.emptyTracklist   = True
-        self.isMainWinVisible = True
-        # GTK+ handlers
-        self.statusIcon.connect('activate',     self.toggleWinVisibility)
-        self.statusIcon.connect('popup-menu',   self.onPopupMenu)
-        self.statusIcon.connect('size-changed', self.renderIcons)
-        # Install everything
-        self.statusIcon.set_tooltip(consts.appName)
-        self.onNewTrack(None)
-        self.statusIcon.set_visible(True)
-
-
-    def uninstall(self):
-        """ Uninstall the Status icon """
-        self.statusIcon.set_visible(False)
+        modules.Module.__init__(self, handlers)
 
 
     def renderIcons(self, statusIcon, availableSize):
@@ -86,32 +69,6 @@ class StatusIcon(modules.Module):
         else:             statusIcon.set_from_pixbuf(self.icoNormal)
 
 
-    def onNewTrack(self, track):
-        """ A new track is being played, None if none """
-        if track is None: self.tooltip = consts.appName
-        else:             self.tooltip  = '%s - %s' % (track.getArtist(), track.getTitle())
-
-        self.isPaused  = False
-        self.isPlaying = track is not None
-
-        self.statusIcon.set_from_pixbuf(self.icoNormal)
-        self.statusIcon.set_tooltip(self.tooltip)
-
-
-    def onPause(self):
-        """ The current track has been paused """
-        self.isPaused = True
-        self.statusIcon.set_from_pixbuf(self.icoPause)
-        self.statusIcon.set_tooltip(_('%(tooltip)s [paused]') % {'tooltip': self.tooltip})
-
-
-    def onUnpause(self):
-        """ The current track has been unpaused """
-        self.isPaused = False
-        self.statusIcon.set_from_pixbuf(self.icoNormal)
-        self.statusIcon.set_tooltip(self.tooltip)
-
-
     def toggleWinVisibility(self, statusIcon):
         """ Show/hide the main window """
         if not self.isMainWinVisible:
@@ -125,20 +82,94 @@ class StatusIcon(modules.Module):
             self.mainWindow.show()
 
 
-    # --== Message handler ==--
+    # --== Message handlers ==--
 
 
-    def handleMsg(self, msg, params):
-        """ Handle messages sent to this module """
-        if   msg == consts.MSG_EVT_PAUSED:        self.onPause()
-        elif msg == consts.MSG_EVT_STOPPED:       self.onNewTrack(None)
-        elif msg == consts.MSG_EVT_UNPAUSED:      self.onUnpause()
-        elif msg == consts.MSG_EVT_NEW_TRACK:     self.onNewTrack(params['track'])
-        elif msg == consts.MSG_EVT_MOD_LOADED:    self.install()
-        elif msg == consts.MSG_EVT_TRACK_MOVED:   self.trackHasNext, self.trackHasPrev = params['hasNext'], params['hasPrevious']
-        elif msg == consts.MSG_EVT_APP_STARTED:   self.install()
-        elif msg == consts.MSG_EVT_MOD_UNLOADED:  self.uninstall()
-        elif msg == consts.MSG_EVT_NEW_TRACKLIST: self.emptyTracklist = (len(params['tracks']) == 0)
+    def onModLoaded(self):
+        """ Install the Status icon """
+        self.volume           = 0
+        self.tooltip          = consts.appName
+        self.isPaused         = False
+        self.icoPause         = None
+        self.popupMenu        = None
+        self.isPlaying        = False
+        self.icoNormal        = None
+        self.mainWindow       = prefs.getWidgetsTree().get_widget('win-main')
+        self.trackHasNext     = False
+        self.trackHasPrev     = False
+        self.emptyTracklist   = True
+        self.isMainWinVisible = True
+        # The status icon does not support RGBA, so make sure to use the RGB color map when creating it
+        gtk.widget_push_colormap(self.mainWindow.get_screen().get_rgb_colormap())
+        self.statusIcon = gtk.StatusIcon()
+        gtk.widget_pop_colormap()
+        # GTK+ handlers
+        self.statusIcon.connect('activate',           self.toggleWinVisibility)
+        self.statusIcon.connect('popup-menu',         self.onPopupMenu)
+        self.statusIcon.connect('size-changed',       self.renderIcons)
+        self.statusIcon.connect('scroll-event',       self.onScroll)
+        self.statusIcon.connect('button-press-event', self.onButtonPressed)
+        # Install everything
+        self.statusIcon.set_tooltip(consts.appName)
+        self.onNewTrack(None)
+        self.statusIcon.set_visible(True)
+
+
+    def onModUnloaded(self):
+        """ Uninstall the Status icon """
+        self.statusIcon.set_visible(False)
+        self.statusIcon = None
+        if not self.isMainWinVisible:
+            self.mainWindow.show()
+            self.isMainWinVisible = True
+
+
+    def onNewTrack(self, track):
+        """ A new track is being played, None if none """
+        if track is None: self.tooltip = consts.appName
+        else:             self.tooltip  = '%s - %s' % (track.getArtist(), track.getTitle())
+
+        self.isPaused  = False
+        self.isPlaying = track is not None
+
+        self.statusIcon.set_from_pixbuf(self.icoNormal)
+        self.statusIcon.set_tooltip(self.tooltip)
+
+
+    def onPaused(self):
+        """ The current track has been paused """
+        self.isPaused = True
+        self.statusIcon.set_from_pixbuf(self.icoPause)
+        self.statusIcon.set_tooltip(_('%(tooltip)s [paused]') % {'tooltip': self.tooltip})
+
+
+    def onUnpaused(self):
+        """ The current track has been unpaused """
+        self.isPaused = False
+        self.statusIcon.set_from_pixbuf(self.icoNormal)
+        self.statusIcon.set_tooltip(self.tooltip)
+
+
+    def onTrackMoved(self, hasPrevious, hasNext):
+        """ The position of the current track in the playlist has changed """
+        self.trackHasNext = hasNext
+        self.trackHasPrev = hasPrevious
+
+
+    def onVolumeChanged(self, value):
+        """ The volume has changed """
+        self.volume = value
+
+
+    def onNewTracklist(self, tracks, playtime):
+        """ A new tracklist has been defined """
+        if len(tracks) == 0: self.emptyTracklist = True
+        else:                self.emptyTracklist = False
+
+
+    def onStopped(self):
+        """ The playback has been stopped """
+        self.onNewTrack(None)
 
 
     # --== GTK handlers ==--
@@ -173,3 +204,19 @@ class StatusIcon(modules.Module):
         self.menuPlay.set_sensitive((not (self.isPlaying or self.emptyTracklist)) or self.isPaused)
 
         self.popupMenu.popup(None, None, gtk.status_icon_position_menu, button, time, statusIcon)
+
+
+    def onScroll(self, statusIcon, scrollEvent):
+        """ The mouse is scrolled on the status icon """
+        if scrollEvent.direction == gtk.gdk.SCROLL_UP or scrollEvent.direction == gtk.gdk.SCROLL_RIGHT:
+            self.volume = min(1.0, self.volume + 0.05)
+        else:
+            self.volume = max(0.0, self.volume - 0.05)
+
+        modules.postMsg(consts.MSG_CMD_SET_VOLUME, {'value': self.volume})
+
+
+    def onButtonPressed(self, statusIcon, buttonEvent):
+        """ A button is pressed on the status icon """
+        if buttonEvent.button == 2:
+            modules.postMsg(consts.MSG_CMD_TOGGLE_PAUSE)
