@@ -16,12 +16,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import cgi, gobject, gtk, modules, tools
+import gobject, gtk, modules, os.path, tools
 
 from tools   import consts
 from gettext import gettext as _
 
-MOD_INFO = ('Track Panel', 'Track Panel', '', [], True, False)
+MOD_INFO = ('Track Panel', 'Track Panel', '', [], True, False, consts.MODCAT_NONE)
 
 
 class TrackPanel(modules.Module):
@@ -32,12 +32,19 @@ class TrackPanel(modules.Module):
 
     def __init__(self):
         """ Constructor """
-        modules.Module.__init__(self, (consts.MSG_EVT_APP_STARTED, consts.MSG_EVT_NEW_TRACK, consts.MSG_EVT_STOPPED, consts.MSG_CMD_SET_COVER))
+        handlers = {
+                        consts.MSG_EVT_STOPPED:     self.onStopped,
+                        consts.MSG_CMD_SET_COVER:   self.onSetCover,
+                        consts.MSG_EVT_NEW_TRACK:   self.onNewTrack,
+                        consts.MSG_EVT_APP_STARTED: self.onAppStarted,
+                   }
+
+        modules.Module.__init__(self, handlers)
 
 
     def __setTitle(self, title, length=None):
         """ Change the title of the current track """
-        title = cgi.escape(title)
+        title = tools.htmlEscape(title)
 
         if length is None: self.txtTitle.set_markup('<span size="larger"><b>%s</b></span>' % title)
         else:              self.txtTitle.set_markup('<span size="larger"><b>%s</b></span>  [%s]' % (title, tools.sec2str(length)))
@@ -48,12 +55,8 @@ class TrackPanel(modules.Module):
             Change the current image to imgPath.
             Use the application's icon if imgPath is None.
         """
-        if imgPath is None:
-            self.img.set_from_file(consts.fileImgIcon64)
-            self.imgFrame.set_shadow_type(gtk.SHADOW_NONE)
-        else:
-            self.img.set_from_file(imgPath)
-            self.imgFrame.set_shadow_type(gtk.SHADOW_IN)
+        if imgPath is None: self.img.set_from_file(os.path.join(tools.consts.dirPix, 'cover-none.png'))
+        else:               self.img.set_from_file(imgPath)
 
 
     def __showCover(self, x, y):
@@ -90,6 +93,9 @@ class TrackPanel(modules.Module):
         self.coverWindow.show_all()
 
 
+    # --== Message handlers ==--
+
+
     def onAppStarted(self):
         """ Real initialization function, called when this module has been loaded """
         # Widgets
@@ -104,10 +110,7 @@ class TrackPanel(modules.Module):
         self.coverTimerId      = None
         self.currCoverPath     = None
         self.lastMousePosition = (0, 0)
-        # Initial state
-        self.onStopped()
         # GTK handlers
-        evtBox.connect('button-press-event', self.onImgButtonPressed)
         evtBox.connect('leave-notify-event', self.onImgMouseLeave)
         evtBox.connect('enter-notify-event', self.onImgMouseEnter)
 
@@ -127,18 +130,20 @@ class TrackPanel(modules.Module):
 
         self.__setImage(None)
         self.__setTitle(consts.appName)
-        self.txtMisc.set_text('...And Music For All')
+        self.txtMisc.set_text('...And Music For All\n')
+
+
+    def onSetCover(self, track, pathThumbnail, pathFullSize):
+        """ Set the cover that is currently displayed """
+        # Must check if currTrack is not None, because '==' calls the cmp() method and this fails on None
+        if self.currTrack is not None and track == self.currTrack:
+            self.currCoverPath = pathFullSize
+            self.__setImage(pathThumbnail)
 
 
     # --== GTK handlers ==--
 
 
-    def onImgButtonPressed(self, evtbox, event):
-        """ Show the larger version of the current cover, if any """
-        if event.button == 1 and event.type == gtk.gdk.BUTTON_PRESS and self.currCoverPath is not None:
-            self.__showCover(event.x_root, event.y_root)
-
-
     def onImgMouseEnter(self, evtBox, event):
         """ The mouse is over the event box """
         if self.currCoverPath is not None and (event.x_root, event.y_root) != self.lastMousePosition:
@@ -166,20 +171,3 @@ class TrackPanel(modules.Module):
             self.coverWindow.destroy()
             self.coverWindow = None
             self.lastMousePosition = tools.getCursorPosition()
-
-
-   # --== Message handler ==--
-
-
-    def handleMsg(self, msg, params):
-        """ Handle messages sent to this module """
-        if msg == consts.MSG_CMD_SET_COVER:
-            if params['track'] == self.currTrack:
-                self.currCoverPath = params['pathFullSize']
-                self.__setImage(params['pathThumbnail'])
-        elif msg == consts.MSG_EVT_NEW_TRACK:
-            self.onNewTrack(params['track'])
-        elif msg == consts.MSG_EVT_STOPPED:
-            self.onStopped()
-        elif msg == consts.MSG_EVT_APP_STARTED:
-            self.onAppStarted()
