@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Author: Ingelrest François (Athropos@gmail.com)
+# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import gobject, gtk, gui.preferences, os, Queue, sys, threading, tools, traceback
+import gobject, gtk, gui, gui.preferences, os, sys, threading, traceback
 
 from tools   import consts, log, prefs
 from gettext import gettext as _
@@ -29,8 +29,9 @@ from gettext import gettext as _
     MODINFO_DESC,           # Description of the module, translated into the current locale
     MODINFO_DEPS,           # A list of special Python dependencies (e.g., pynotify)
     MODINFO_MANDATORY,      # True if the module cannot be disabled
-    MODINFO_CONFIGURABLE    # True if the module can be configured
-) = range(6)
+    MODINFO_CONFIGURABLE,   # True if the module can be configured
+    MODINFO_CATEGORY,       # Category the module belongs to
+) = range(7)
 
 
 # Values associated with a module
@@ -83,7 +84,12 @@ def load(name):
     try:
         module[MOD_INSTANCE] = getattr(module[MOD_PMODULE], module[MOD_CLASSNAME])()
         module[MOD_INSTANCE].start()
-        module[MOD_INSTANCE].postMsg(consts.MSG_EVT_MOD_LOADED)
+
+        mHandlersLock.acquire()
+        if module[MOD_INSTANCE] in mHandlers[consts.MSG_EVT_MOD_LOADED]:
+            module[MOD_INSTANCE].postMsg(consts.MSG_EVT_MOD_LOADED)
+        mHandlersLock.release()
+
         log.logger.info('Module loaded: %s' % module[MOD_CLASSNAME])
         mEnabledModules.append(name)
         prefs.set(__name__, 'enabled_modules', mEnabledModules)
@@ -163,56 +169,176 @@ def postQuitMsg():
     gobject.idle_add(__postQuitMsg)
 
 
+mMenuItems  = {}
+mSeparator  = None
+mAccelGroup = None
+
+
+def __addMenuItem(label, callback, accelerator):
+    """ This is the 'real' addMenuItem function, which must be executed in the GTK main loop """
+    global mAccelGroup, mSeparator
+
+    menu = prefs.getWidgetsTree().get_widget('menu-edit')
+
+    # Remove all menu items
+    if len(mMenuItems) != 0:
+        menu.remove(mSeparator)
+        for menuitem in mMenuItems.itervalues():
+            menu.remove(menuitem)
+
+    # Create a new menu item for the module
+    menuitem = gtk.MenuItem(label)
+    menuitem.connect('activate', callback)
+    menuitem.show()
+    mMenuItems[label] = menuitem
+
+    # Add an accelerator if needed
+    if accelerator is not None:
+        if mAccelGroup is None:
+            mAccelGroup = gtk.AccelGroup()
+            prefs.getWidgetsTree().get_widget('win-main').add_accel_group(mAccelGroup)
+
+        key, mod = gtk.accelerator_parse(accelerator)
+        menuitem.add_accelerator('activate', mAccelGroup, key, mod, gtk.ACCEL_VISIBLE)
+
+    # Create the separator?
+    if mSeparator is None:
+        mSeparator = gtk.SeparatorMenuItem()
+        mSeparator.show()
+
+    # Re-add items alphabetically, including the new one
+    menu.insert(mSeparator, 0)
+    for item in sorted(mMenuItems.items(), key = lambda item: item[0], reverse = True):
+        menu.insert(item[1], 0)
+
+
+def addMenuItem(label, callback, accelerator=None):
+    """ Add a menu item to the 'modules' menu """
+    gobject.idle_add(__addMenuItem, label, callback, accelerator)
+
+
+def __delMenuItem(label):
+    """ This is the 'real' delMenuItem function, which must be executed in the GTK main loop """
+    # Make sure the menu item is there
+    if label not in mMenuItems:
+        return
+
+    menu = prefs.getWidgetsTree().get_widget('menu-edit')
+
+    # Remove all current menu items
+    menu.remove(mSeparator)
+    for menuitem in mMenuItems.itervalues():
+        menu.remove(menuitem)
+
+    # Delete the given menu item
+    del mMenuItems[label]
+
+    # Re-add items if needed
+    if len(mMenuItems) != 0:
+        menu.insert(mSeparator, 0)
+        for item in sorted(mMenuItems.items(), key = lambda item: item[0], reverse = True):
+            menu.insert(item[1], 0)
+
+
+def delMenuItem(label):
+    """ Delete a menu item from the 'modules' menu """
+    gobject.idle_add(__delMenuItem, label)
+
+
 # --== Base classes for modules ==--
 
 
 class ModuleBase:
     """ This class makes sure that all modules have some mandatory functions """
-    def join(self):                   pass
-    def start(self):                  pass
-    def configure(self, parent):      pass
-    def handleMsg(self, msg, params): pass
+
+    def join(self):
+        pass
+
+    def start(self):
+        pass
+
+    def configure(self, parent):
+        pass
+
+    def handleMsg(self, msg, params):
+        pass
+
+    def restartRequired(self):
+        gobject.idle_add(gui.infoMsgBox, None, _('Restart required'), _('You must restart the application for this modification to take effect.'))
+
 
 
 class Module(ModuleBase):
     """ This is the base class for non-threaded modules """
-    def __init__(self, messages):      register(self, messages)
-    def postMsg(self, msg, params={}): gobject.idle_add(self.handleMsg, msg, params)
+
+    def __init__(self, handlers):
+        self.handlers = handlers
+        register(self, handlers.keys())
+
+    def postMsg(self, msg, params={}):
+        gobject.idle_add(self.__dispatch, msg, params)
+
+    def __dispatch(self, msg, params):
+        self.handlers[msg](**params)
+
 
 
 class ThreadedModule(threading.Thread, ModuleBase):
     """ This is the base class for threaded modules """
 
-    def __init__(self, messages):
+    def __init__(self, handlers):
         """ Constructor """
-        threading.Thread.__init__(self)
-        register(self, messages + (consts.MSG_EVT_APP_QUIT, consts.MSG_EVT_MOD_UNLOADED))
+        import Queue
+
         # Attributes
         self.queue        = Queue.Queue(0)            # List of queued messages
         self.gtkResult    = None                      # Value returned by the function executed in the GTK loop
         self.gtkSemaphore = threading.Semaphore(0)    # Used to execute some code in the GTK loop
 
+        # Initialization
+        threading.Thread.__init__(self)
+
+        # Add QUIT and UNLOADED messages if needed
+        # These messages are required to exit the thread's loop
+        if consts.MSG_EVT_APP_QUIT not in handlers:     handlers[consts.MSG_EVT_APP_QUIT]     = lambda: None
+        if consts.MSG_EVT_MOD_UNLOADED not in handlers: handlers[consts.MSG_EVT_MOD_UNLOADED] = lambda: None
+
+        self.handlers = handlers
+        register(self, handlers.keys())
+
     def __gtkExecute(self, func):
         """ Private function, must be executed in the GTK main loop """
         self.gtkResult = func()
         self.gtkSemaphore.release()
 
     def gtkExecute(self, func):
-        """ Execute func in the GTK main loop, and block the execution until done """
+        """ Execute func in the GTK main loop, and block the execution of the thread until done """
         gobject.idle_add(self.__gtkExecute, func)
         self.gtkSemaphore.acquire()
         return self.gtkResult
 
+    def threadExecute(self, func, *args):
+        """
+            Schedule func(*args) to be called by the thread
+            This is used to avoid func to be executed in the GTK main loop
+        """
+        self.postMsg(consts.MSG_CMD_THREAD_EXECUTE, (func, args))
+
     def postMsg(self, msg, params={}):
         """ Enqueue a message in this threads's message queue """
         self.queue.put((msg, params))
 
     def run(self):
-        """ Wait for messages and pass them to handleMsg() """
+        """ Wait for messages and handle them """
         msg = None
         while msg != consts.MSG_EVT_APP_QUIT and msg != consts.MSG_EVT_MOD_UNLOADED:
             (msg, params) = self.queue.get(True)
-            self.handleMsg(msg, params)
+
+            if msg == consts.MSG_CMD_THREAD_EXECUTE:
+                (func, args) = params
+                func(*args)
+            else:
+                self.handlers[msg](**params)
 
 
 # --== Entry point ==--
@@ -227,7 +353,6 @@ mEnabledModules = prefs.get(__name__, 'enabled_modules', [])                   #
 
 
 # Find modules, instantiate those that are mandatory or that have been previously enabled by the user
-# This code should not be executed automatically, to prevent the remote from doing it when importing this module
 sys.path.append(mModDir)
 for file in [os.path.splitext(file)[0] for file in os.listdir(mModDir) if file.endswith('.py') and file != '__init__.py']:
     try:
